""" Unit tests for the UDP socket classes """

import unittest
import unittest.mock
import time
import socket

from context import (  # type: ignore
    GeckoUdpProtocolHandler,
    GeckoUdpSocket,
)


class MockSocket:
    """A class that looks like a socket, but doesn't really
    send or receive anything - used in the tests"""

    def __init__(self):
        self.data_for_recvfrom = None
        self.last_data_sent = None

    def close(self):
        pass

    def settimeout(self, timeout):
        pass

    def recvfrom(self, max_size):
        if self.data_for_recvfrom is None:
            raise socket.timeout
        data = self.data_for_recvfrom
        self.data_for_recvfrom = None
        return (data, ("Mock", "Port"))

    def sendto(self, data, dest):
        self.last_data_sent = data


class MockHandler(GeckoUdpProtocolHandler):
    def can_handle(self, received_bytes: bytes, sender: tuple) -> bool:
        return True

    def handle(
        self, socket: GeckoUdpSocket, received_bytes: bytes, sender: tuple
    ) -> None:
        pass


class TestUdpProtocolHandler(unittest.TestCase):
    def setUp(self):
        self.base_handler = MockHandler()

    def test_constructor(self):
        self.assertIsNotNone(self.base_handler)

    @unittest.expectedFailure
    def test_default_bytes_not_implemented(self):
        self.assertEqual(self.base_handler.send_bytes, None)

    def test_loop(self):
        self.base_handler.loop(None)

    def test_not_timedout(self):
        self.assertFalse(self.base_handler.has_timedout)

    def test_timedout(self):
        handler = MockHandler(timeout=0.005)
        self.assertFalse(handler.has_timedout)
        self.assertFalse(handler.should_remove_handler)
        time.sleep(0.005)
        self.assertTrue(handler.has_timedout)
        self.assertFalse(handler.should_remove_handler)
        handler.loop(None)
        # Still false since we don't know if the sub-class wants to do this
        self.assertFalse(handler.should_remove_handler)

    def test_should_remove_handler(self):
        self.assertFalse(self.base_handler.should_remove_handler)


class TestGeckoUdpSocket(unittest.TestCase):
    """Test the GeckoUdpSocket class"""

    def setUp(self):
        class UdpSocketHelper(GeckoUdpSocket):
            def __init__(self):
                self.mock_socket = MockSocket()
                super().__init__(self.mock_socket)

            def wait(self):
                while self.isbusy:
                    pass

        self.socket = UdpSocketHelper()
        self.socket.open()

    def tearDown(self):
        self.socket.close()

    def test_construct(self):
        socket = GeckoUdpSocket()
        self.assertFalse(socket.isopen)
        self.assertTrue(self.socket.isopen)

    def test_send(self):
        class Send(MockHandler):
            @property
            def send_bytes(self):
                return b"REQUEST"

        self.socket.queue_send(Send(), (1, 2))
        self.assertTrue(self.socket.isbusy)
        self.socket.wait()
        self.assertEqual(self.socket.mock_socket.last_data_sent, b"REQUEST")

    def test_receive(self):
        class Receive(GeckoUdpProtocolHandler):
            def __init__(self):
                super().__init__()
                self.received = None

            def can_handle(self, received_bytes: bytes, sender: tuple):
                return True

            def handle(self, received_bytes: bytes, sender: tuple):
                self.received = received_bytes
                self._should_remove_handler = True

        self.socket.mock_socket.data_for_recvfrom = None
        handler = Receive()
        self.assertIsNone(handler.received)
        try:
            self.socket.add_receive_handler(handler)
            self.socket.mock_socket.data_for_recvfrom = "ONCE"
            while handler.received is None:
                pass
        finally:
            self.socket.wait()
            self.assertEqual(handler.received, "ONCE")

    def test_receive_retry(self):
        class Retry(GeckoUdpProtocolHandler):
            def __init__(self):
                super().__init__(
                    timeout=0.01, retry_count=5, on_retry_failed=self._on_retry_failed
                )
                self.retry_occurred = 0

            @property
            def send_bytes(self):
                return b"REQUEST"

            def can_handle(self, received_bytes: bytes, sender: tuple):
                return True

            def handle(self, received_bytes: bytes, sender: tuple):
                self._should_remove_handler = True

            def _reset_timeout(self):
                super()._reset_timeout()
                self.retry_occurred += 1

            def _on_retry_failed(self, handler, socket):
                handler._should_remove_handler = True

        self.socket.mock_socket.data_for_recvfrom = None
        retry_handler = Retry()
        retry_handler.last_destination = (1, 2)
        now = time.monotonic()

        try:
            self.socket.add_receive_handler(retry_handler)
            while retry_handler.retry_occurred < 5 and (time.monotonic() - now) < 5:
                pass
            while len(self.socket._receive_handlers) > 0:
                pass
        finally:
            self.socket.wait()
            self.assertListEqual(self.socket._receive_handlers, [])
            self.assertEqual(retry_handler.retry_occurred, 5)


if __name__ == "__main__":
    unittest.main()



#!/usr/bin/python3
"""
    GeckoConfigStruct - A class to manage the ConfigStruct for 'InXE v7'
"""

from . import (
    GeckoByteStructAccessor,
    GeckoWordStructAccessor,
    GeckoTimeStructAccessor,
    GeckoBoolStructAccessor,
    GeckoEnumStructAccessor,
    GeckoTempStructAccessor,
)


class GeckoConfigStruct:
    def __init__(self, struct_):
        self.struct = struct_

    @property
    def version(self):
        return 7

    @property
    def output_keys(self):
        return []

    @property
    def accessors(self):
        return {
            "ConfigNumber": GeckoByteStructAccessor(
                self.struct, "ConfigNumber", 0, "ALL"
            ),
            "OutHtRCur": GeckoByteStructAccessor(self.struct, "OutHtRCur", 27, "ALL"),
            "DirectCur": GeckoByteStructAccessor(self.struct, "DirectCur", 26, "ALL"),
            "PumpTimeOut": GeckoByteStructAccessor(
                self.struct, "PumpTimeOut", 31, "ALL"
            ),
            "LightTimeOut": GeckoByteStructAccessor(
                self.struct, "LightTimeOut", 32, "ALL"
            ),
            "FiltInterface": GeckoEnumStructAccessor(
                self.struct,
                "FiltInterface",
                17,
                None,
                ["PurgeOnly", "FiltCP", "FiltP1"],
                None,
                None,
                "ALL",
            ),
            "OTActSetpointG": GeckoTempStructAccessor(
                self.struct, "OTActSetpointG", 33, "ALL"
            ),
            "OTTriggerG": GeckoByteStructAccessor(self.struct, "OTTriggerG", 35, "ALL"),
            "CPOTMaxOnTime": GeckoWordStructAccessor(
                self.struct, "CPOTMaxOnTime", 36, "ALL"
            ),
            "CPOTMaxOffTime": GeckoWordStructAccessor(
                self.struct, "CPOTMaxOffTime", 38, "ALL"
            ),
            "FiltOTDuration24H": GeckoWordStructAccessor(
                self.struct, "FiltOTDuration24H", 40, "ALL"
            ),
            "FiltSuspendTime": GeckoByteStructAccessor(
                self.struct, "FiltSuspendTime", 42, "ALL"
            ),
            "O3Pump": GeckoEnumStructAccessor(
                self.struct, "O3Pump", 14, None, ["CP", "P1"], None, None, "ALL"
            ),
            "O3Type": GeckoEnumStructAccessor(
                self.struct,
                "O3Type",
                15,
                None,
                ["Standard", "Toggle"],
                None,
                None,
                "ALL",
            ),
            "O3SuspendTime": GeckoByteStructAccessor(
                self.struct, "O3SuspendTime", 43, "ALL"
            ),
            "SetpointG": GeckoTempStructAccessor(self.struct, "SetpointG", 1, "ALL"),
            "HeaterPump": GeckoEnumStructAccessor(
                self.struct, "HeaterPump", 16, None, ["CP", "P1"], None, None, "ALL"
            ),
            "TempUnits": GeckoEnumStructAccessor(
                self.struct, "TempUnits", 18, None, ["F", "C"], None, None, "ALL"
            ),
            "MinSetpointG": GeckoTempStructAccessor(
                self.struct, "MinSetpointG", 46, "ALL"
            ),
            "MaxSetpointG": GeckoTempStructAccessor(
                self.struct, "MaxSetpointG", 48, "ALL"
            ),
            "NbPhases": GeckoByteStructAccessor(self.struct, "NbPhases", 29, "ALL"),
            "InputCurrent": GeckoByteStructAccessor(
                self.struct, "InputCurrent", 30, "ALL"
            ),
            "FiltFreq": GeckoByteStructAccessor(self.struct, "FiltFreq", 4, "ALL"),
            "FiltStart": GeckoTimeStructAccessor(self.struct, "FiltStart", 5, "ALL"),
            "FiltDur": GeckoTimeStructAccessor(self.struct, "FiltDur", 3, "ALL"),
            "EconStart": GeckoTimeStructAccessor(self.struct, "EconStart", 7, "ALL"),
            "EconDur": GeckoTimeStructAccessor(self.struct, "EconDur", 8, "ALL"),
            "EconProgAvailable": GeckoEnumStructAccessor(
                self.struct,
                "EconProgAvailable",
                50,
                None,
                ["NA", "Available"],
                None,
                None,
                "ALL",
            ),
            "SoakOnCustomKey": GeckoBoolStructAccessor(
                self.struct, "SoakOnCustomKey", 51, 0, "ALL"
            ),
            "OffOnCustomKey": GeckoBoolStructAccessor(
                self.struct, "OffOnCustomKey", 51, 1, "ALL"
            ),
            "EconControlableManually": GeckoBoolStructAccessor(
                self.struct, "EconControlableManually", 51, 2, "ALL"
            ),
            "TimeFormat": GeckoEnumStructAccessor(
                self.struct,
                "TimeFormat",
                19,
                None,
                ["NA", "AmPm", "24h"],
                None,
                None,
                "ALL",
            ),
            "AmbiantOHTrigADC": GeckoWordStructAccessor(
                self.struct, "AmbiantOHTrigADC", 44, "ALL"
            ),
        }



#!/usr/bin/python3
"""
    GeckoConfigStruct - A class to manage the ConfigStruct for 'inYE-V3 v84'
"""

from . import (
    GeckoByteStructAccessor,
    GeckoWordStructAccessor,
    GeckoTimeStructAccessor,
    GeckoBoolStructAccessor,
    GeckoEnumStructAccessor,
    GeckoTempStructAccessor,
)


class GeckoConfigStruct:
    def __init__(self, struct_):
        self.struct = struct_

    @property
    def version(self):
        return 84

    @property
    def output_keys(self):
        return [
            "Out1",
            "Out2",
            "Out3",
            "Out4",
            "Out5",
            "OutHtr",
            "HeatPumpCurrent",
            "HeatPumpFuse",
            "Out6",
            "Out7",
            "Out8",
            "Out9",
            "Out10",
            "Out11",
            "Out12",
            "Direct",
            "Direct2",
            "OutLi",
            "LightActivationSafety",
        ]

    @property
    def accessors(self):
        return {
            "ConfigNumber": GeckoByteStructAccessor(
                self.struct, "ConfigNumber", 0, "ALL"
            ),
            "Out1": GeckoEnumStructAccessor(
                self.struct,
                "Out1",
                12,
                None,
                [
                    "NA",
                    "P1H",
                    "P1L",
                    "P2H",
                    "P2L",
                    "P3H",
                    "P3L",
                    "P4H",
                    "P4L",
                    "P5",
                    "BLO",
                    "CP",
                    "O3",
                    "L120",
                    "HTR",
                    "",
                    "Fan",
                    "",
                    "",
                    "FullOn",
                    "",
                    "",
                    "",
                    "ONZEN",
                    "Protection",
                    "HTR2",
                    "Waterfall",
                    "AUX",
                ],
                None,
                None,
                "ALL",
            ),
            "Out2": GeckoEnumStructAccessor(
                self.struct,
                "Out2",
                13,
                None,
                [
                    "NA",
                    "P1H",
                    "P1L",
                    "P2H",
                    "P2L",
                    "P3H",
                    "P3L",
                    "P4H",
                    "P4L",
                    "P5",
                    "BLO",
                    "CP",
                    "O3",
                    "L120",
                    "HTR",
                    "",
                    "Fan",
                    "",
                    "",
                    "FullOn",
                    "",
                    "",
                    "",
                    "ONZEN",
                    "Protection",
                    "HTR2",
                    "Waterfall",
                    "AUX",
                ],
                None,
                None,
                "ALL",
            ),
            "Out3": GeckoEnumStructAccessor(
                self.struct,
                "Out3",
                14,
                None,
                [
                    "NA",
                    "P1H",
                    "P1L",
                    "P2H",
                    "P2L",
                    "P3H",
                    "P3L",
                    "P4H",
                    "P4L",
                    "P5",
                    "BLO",
                    "CP",
                    "O3",
                    "L120",
                    "HTR",
                    "",
                    "Fan",
                    "",
                    "",
                    "FullOn",
                    "",
                    "",
                    "",
                    "ONZEN",
                    "Protection",
                    "HTR2",
                    "Waterfall",
                    "AUX",
                ],
                None,
                None,
                "ALL",
            ),
            "Out4": GeckoEnumStructAccessor(
                self.struct,
                "Out4",
                15,
                None,
                [
                    "NA",
                    "P1H",
                    "P1L",
                    "P2H",
                    "P2L",
                    "P3H",
                    "P3L",
                    "P4H",
                    "P4L",
                    "P5",
                    "BLO",
                    "CP",
                    "O3",
                    "L120",
                    "HTR",
                    "",
                    "Fan",
                    "",
                    "",
                    "FullOn",
                    "",
                    "",
                    "",
                    "ONZEN",
                    "Protection",
                    "HTR2",
                    "Waterfall",
                    "AUX",
                ],
                None,
                None,
                "ALL",
            ),
            "Out5": GeckoEnumStructAccessor(
                self.struct,
                "Out5",
                16,
                None,
                [
                    "NA",
                    "P1H",
                    "P1L",
                    "P2H",
                    "P2L",
                    "P3H",
                    "P3L",
                    "P4H",
                    "P4L",
                    "P5",
                    "BLO",
                    "CP",
                    "O3",
                    "L120",
                    "HTR",
                    "",
                    "Fan",
                    "",
                    "",
                    "FullOn",
                    "",
                    "",
                    "",
                    "ONZEN",
                    "Protection",
                    "HTR2",
                    "Waterfall",
                    "AUX",
                ],
                None,
                None,
                "ALL",
            ),
            "OutHtr": GeckoEnumStructAccessor(
                self.struct,
                "OutHtr",
                26,
                None,
                [
                    "NA",
                    "P1H",
                    "",
                    "",
                    "",
                    "",
                    "",
                    "",
                    "",
                    "",
                    "",
                    "",
                    "",
                    "",
                    "HTR",
                    "",
                    "",
                    "",
                    "",
                    "",
                    "",
                    "",
                    "",
                    "",
                    "",
                    "HTR2",
                ],
                None,
                None,
                "ALL",
            ),
            "Out1Cur": GeckoByteStructAccessor(self.struct, "Out1Cur", 36, "ALL"),
            "Out2Cur": GeckoByteStructAccessor(self.struct, "Out2Cur", 37, "ALL"),
            "Out3Cur": GeckoByteStructAccessor(self.struct, "Out3Cur", 38, "ALL"),
            "Out4Cur": GeckoByteStructAccessor(self.struct, "Out4Cur", 39, "ALL"),
            "Out5Cur": GeckoByteStructAccessor(self.struct, "Out5Cur", 40, "ALL"),
            "OutHtRCur": GeckoByteStructAccessor(self.struct, "OutHtRCur", 50, "ALL"),
            "HeatPumpCurrent": GeckoEnumStructAccessor(
                self.struct,
                "HeatPumpCurrent",
                161,
                None,
                [
                    "Not_Set",
                    "0A",
                    "1A",
                    "2A",
                    "3A",
                    "4A",
                    "5A",
                    "6A",
                    "7A",
                    "8A",
                    "9A",
                    "10A",
                    "11A",
                    "12A",
                    "13A",
                    "14A",
                    "15A",
                ],
                None,
                None,
                "ALL",
            ),
            "HeatPumpFuse": GeckoEnumStructAccessor(
                self.struct,
                "HeatPumpFuse",
                163,
                None,
                ["F1", "F2", "F3", "F21", "F22", "F23", "Line1", "Line2", "Line3"],
                None,
                None,
                "ALL",
            ),
            "Out6": GeckoEnumStructAccessor(
                self.struct,
                "Out6",
                17,
                None,
                [
                    "NA",
                    "P1H",
                    "P1L",
                    "P2H",
                    "P2L",
                    "P3H",
                    "P3L",
                    "P4H",
                    "P4L",
                    "P5",
                    "BLO",
                    "CP",
                    "O3",
                    "L120",
                    "HTR",
                    "",
                    "Fan",
                    "",
                    "",
                    "FullOn",
                    "",
                    "",
                    "",
                    "ONZEN",
                    "Protection",
                    "HTR2",
                    "Waterfall",
                    "AUX",
                ],
                None,
                None,
                "ALL",
            ),
            "Out7": GeckoEnumStructAccessor(
                self.struct,
                "Out7",
                18,
                None,
                [
                    "NA",
                    "P1H",
                    "P1L",
                    "P2H",
                    "P2L",
                    "P3H",
                    "P3L",
                    "P4H",
                    "P4L",
                    "P5",
                    "BLO",
                    "CP",
                    "O3",
                    "L120",
                    "HTR",
                    "",
                    "Fan",
                    "",
                    "",
                    "FullOn",
                    "",
                    "",
                    "",
                    "ONZEN",
                    "Protection",
                    "HTR2",
                    "Waterfall",
                    "AUX",
                ],
                None,
                None,
                "ALL",
            ),
            "Out8": GeckoEnumStructAccessor(
                self.struct,
                "Out8",
                19,
                None,
                [
                    "NA",
                    "P1H",
                    "P1L",
                    "P2H",
                    "P2L",
                    "P3H",
                    "P3L",
                    "P4H",
                    "P4L",
                    "P5",
                    "BLO",
                    "CP",
                    "O3",
                    "L120",
                    "HTR",
                    "",
                    "Fan",
                    "",
                    "",
                    "FullOn",
                    "",
                    "",
                    "",
                    "ONZEN",
                    "Protection",
                    "HTR2",
                    "Waterfall",
                    "AUX",
                ],
                None,
                None,
                "ALL",
            ),
            "Out9": GeckoEnumStructAccessor(
                self.struct,
                "Out9",
                20,
                None,
                [
                    "NA",
                    "P1H",
                    "P1L",
                    "P2H",
                    "P2L",
                    "P3H",
                    "P3L",
                    "P4H",
                    "P4L",
                    "P5",
                    "BLO",
                    "CP",
                    "O3",
                    "L120",
                    "HTR",
                    "",
                    "Fan",
                    "",
                    "",
                    "FullOn",
                    "",
                    "",
                    "",
                    "ONZEN",
                    "Protection",
                    "HTR2",
                    "Waterfall",
                    "AUX",
                ],
                None,
                None,
                "ALL",
            ),
            "Out10": GeckoEnumStructAccessor(
                self.struct,
                "Out10",
                21,
                None,
                [
                    "NA",
                    "P1H",
                    "P1L",
                    "P2H",
                    "P2L",
                    "P3H",
                    "P3L",
                    "P4H",
                    "P4L",
                    "P5",
                    "BLO",
                    "CP",
                    "O3",
                    "L120",
                    "HTR",
                    "",
                    "Fan",
                    "",
                    "",
                    "FullOn",
                    "",
                    "",
                    "",
                    "ONZEN",
                    "Protection",
                    "HTR2",
                    "Waterfall",
                    "AUX",
                ],
                None,
                None,
                "ALL",
            ),
            "Out11": GeckoEnumStructAccessor(
                self.struct,
                "Out11",
                22,
                None,
                [
                    "NA",
                    "P1H",
                    "P1L",
                    "P2H",
                    "P2L",
                    "P3H",
                    "P3L",
                    "P4H",
                    "P4L",
                    "P5",
                    "BLO",
                    "CP",
                    "O3",
                    "L120",
                    "HTR",
                    "",
                    "Fan",
                    "",
                    "",
                    "FullOn",
                    "",
                    "",
                    "",
                    "ONZEN",
                    "Protection",
                    "HTR2",
                    "Waterfall",
                    "AUX",
                ],
                None,
                None,
                "ALL",
            ),
            "Out12": GeckoEnumStructAccessor(
                self.struct,
                "Out12",
                23,
                None,
                [
                    "NA",
                    "P1H",
                    "P1L",
                    "P2H",
                    "P2L",
                    "P3H",
                    "P3L",
                    "P4H",
                    "P4L",
                    "P5",
                    "BLO",
                    "CP",
                    "O3",
                    "L120",
                    "HTR",
                    "",
                    "Fan",
                    "",
                    "",
                    "FullOn",
                    "",
                    "",
                    "",
                    "ONZEN",
                    "Protection",
                    "HTR2",
                    "Waterfall",
                    "AUX",
                ],
                None,
                None,
                "ALL",
            ),
            "Direct": GeckoEnumStructAccessor(
                self.struct,
                "Direct",
                24,
                None,
                ["NA", "", "", "", "", "", "", "", "", "", "", "CP"],
                None,
                None,
                "ALL",
            ),
            "Direct2": GeckoEnumStructAccessor(
                self.struct,
                "Direct2",
                25,
                None,
                ["NA", "", "", "", "", "", "", "", "", "", "", "CP"],
                None,
                None,
                "ALL",
            ),
            "Out6Cur": GeckoByteStructAccessor(self.struct, "Out6Cur", 41, "ALL"),
            "Out7Cur": GeckoByteStructAccessor(self.struct, "Out7Cur", 42, "ALL"),
            "Out8Cur": GeckoByteStructAccessor(self.struct, "Out8Cur", 43, "ALL"),
            "Out9Cur": GeckoByteStructAccessor(self.struct, "Out9Cur", 44, "ALL"),
            "Out10Cur": GeckoByteStructAccessor(self.struct, "Out10Cur", 45, "ALL"),
            "Out11Cur": GeckoByteStructAccessor(self.struct, "Out11Cur", 46, "ALL"),
            "Out12Cur": GeckoByteStructAccessor(self.struct, "Out12Cur", 47, "ALL"),
            "DirectCur": GeckoByteStructAccessor(self.struct, "DirectCur", 48, "ALL"),
            "Direct2Cur": GeckoByteStructAccessor(self.struct, "Direct2Cur", 49, "ALL"),
            "WaterfallAsCP": GeckoBoolStructAccessor(
                self.struct, "WaterfallAsCP", 119, 2, "ALL"
            ),
            "WaterfallValveOnCP": GeckoBoolStructAccessor(
                self.struct, "WaterfallValveOnCP", 119, 4, "ALL"
            ),
            "OutLi": GeckoEnumStructAccessor(
                self.struct,
                "OutLi",
                79,
                None,
                ["", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "LI"],
                None,
                None,
                None,
            ),
            "LightInts": GeckoByteStructAccessor(self.struct, "LightInts", 80, None),
            "LightActivationSafety": GeckoEnumStructAccessor(
                self.struct,
                "LightActivationSafety",
                180,
                None,
                ["Disable", "Enable"],
                None,
                None,
                None,
            ),
            "PumpTimeOut": GeckoByteStructAccessor(
                self.struct, "PumpTimeOut", 54, "ALL"
            ),
            "LightTimeOut": GeckoByteStructAccessor(
                self.struct, "LightTimeOut", 55, "ALL"
            ),
            "L120TimeOut": GeckoByteStructAccessor(
                self.struct, "L120TimeOut", 56, "ALL"
            ),
            "L120Timer": GeckoEnumStructAccessor(
                self.struct, "L120Timer", 81, 0, ["Shared", "Own"], None, 2, "ALL"
            ),
            "WaterfallTimeOut": GeckoByteStructAccessor(
                self.struct, "WaterfallTimeOut", 118, "ALL"
            ),
            "AuxTimeOut": GeckoByteStructAccessor(
                self.struct, "AuxTimeOut", 120, "ALL"
            ),
            "FiltInterface": GeckoEnumStructAccessor(
                self.struct,
                "FiltInterface",
                32,
                None,
                ["PurgeOnly", "FiltCP", "FiltP1", "FiltP1DurOnly"],
                None,
                None,
                "ALL",
            ),
            "CpUsage": GeckoEnumStructAccessor(
                self.struct,
                "CpUsage",
                27,
                None,
                ["STANDARD", "ALWAYS_ON"],
                None,
                None,
                "ALL",
            ),
            "OtOption": GeckoEnumStructAccessor(
                self.struct,
                "OtOption",
                57,
                None,
                ["Disabled", "AlwaysEnabled", "WithSPOver95F"],
                None,
                None,
                "ALL",
            ),
            "PurgeSpeed": GeckoEnumStructAccessor(
                self.struct, "PurgeSpeed", 81, 2, ["Lo", "Hi"], None, 2, "ALL"
            ),
            "AuxAsBubbleGen": GeckoBoolStructAccessor(
                self.struct, "AuxAsBubbleGen", 119, 1, "ALL"
            ),
            "OTTriggerG": GeckoByteStructAccessor(self.struct, "OTTriggerG", 58, "ALL"),
            "CpOnTimeDuringOT": GeckoByteStructAccessor(
                self.struct, "CpOnTimeDuringOT", 59, "ALL"
            ),
            "CpOffTimeDuringOT": GeckoByteStructAccessor(
                self.struct, "CpOffTimeDuringOT", 60, "ALL"
            ),
            "FiltOnTimeDuringOT": GeckoByteStructAccessor(
                self.struct, "FiltOnTimeDuringOT", 61, "ALL"
            ),
            "FiltSuspendTime": GeckoByteStructAccessor(
                self.struct, "FiltSuspendTime", 62, "ALL"
            ),
            "DrainMode": GeckoEnumStructAccessor(
                self.struct,
                "DrainMode",
                78,
                None,
                ["NA", "P1", "CP"],
                None,
                None,
                "ALL",
            ),
            "O3Usage": GeckoEnumStructAccessor(
                self.struct,
                "O3Usage",
                28,
                None,
                ["Filter", "Always"],
                None,
                None,
                "ALL",
            ),
            "O3Pump": GeckoEnumStructAccessor(
                self.struct, "O3Pump", 29, None, ["CP", "P1"], None, None, "ALL"
            ),
            "O3Type": GeckoEnumStructAccessor(
                self.struct,
                "O3Type",
                30,
                None,
                ["Standard", "Toggle"],
                None,
                None,
                "ALL",
            ),
            "O3SuspendTime": GeckoByteStructAccessor(
                self.struct, "O3SuspendTime", 63, "ALL"
            ),
            "SetpointG": GeckoTempStructAccessor(self.struct, "SetpointG", 1, "ALL"),
            "TempUnits": GeckoEnumStructAccessor(
                self.struct, "TempUnits", 33, None, ["F", "C"], None, None, "ALL"
            ),
            "HeaterPump": GeckoEnumStructAccessor(
                self.struct, "HeaterPump", 31, None, ["CP", "P1"], None, None, "ALL"
            ),
            "FlowDetector": GeckoEnumStructAccessor(
                self.struct,
                "FlowDetector",
                164,
                None,
                ["inFlo", "NotInstalled", "PressureSwitch"],
                None,
                None,
                "ALL",
            ),
            "ProbeLocation": GeckoEnumStructAccessor(
                self.struct,
                "ProbeLocation",
                81,
                3,
                ["IntoPiping", "IntoTub"],
                None,
                2,
                "ALL",
            ),
            "CooldownTime": GeckoByteStructAccessor(
                self.struct, "CooldownTime", 35, "ALL"
            ),
            "MinSetpointG": GeckoTempStructAccessor(
                self.struct, "MinSetpointG", 66, "ALL"
            ),
            "MaxSetpointG": GeckoTempStructAccessor(
                self.struct, "MaxSetpointG", 68, "ALL"
            ),
            "EconType": GeckoEnumStructAccessor(
                self.struct,
                "EconType",
                70,
                None,
                ["Standard", "Night"],
                None,
                None,
                "ALL",
            ),
            "NoHeatPeriod": GeckoByteStructAccessor(
                self.struct, "NoHeatPeriod", 77, "ALL"
            ),
            "CoolZoneMode": GeckoEnumStructAccessor(
                self.struct,
                "CoolZoneMode",
                115,
                None,
                [
                    "CHILL",
                    "HEAT_W_BOOST",
                    "HEAT_SAVER",
                    "AUTO_W_BOOST",
                    "AUTO_SAVER",
                    "INTERNAL_HEAT",
                    "BOTH_HEAT",
                ],
                None,
                None,
                "ALL",
            ),
            "GenericHeatPumpID": GeckoByteStructAccessor(
                self.struct, "GenericHeatPumpID", 162, "ALL"
            ),
            "UL_CE": GeckoEnumStructAccessor(
                self.struct, "UL_CE", 51, None, ["UL", "CE"], None, None, "ALL"
            ),
            "MaxNumberOfPhases": GeckoByteStructAccessor(
                self.struct, "MaxNumberOfPhases", 114, "ALL"
            ),
            "NbPhases": GeckoByteStructAccessor(self.struct, "NbPhases", 52, "ALL"),
            "Max1phaseSinglePackCurrent": GeckoByteStructAccessor(
                self.struct, "Max1phaseSinglePackCurrent", 165, "ALL"
            ),
            "Max2phaseSinglePackCurrent": GeckoByteStructAccessor(
                self.struct, "Max2phaseSinglePackCurrent", 166, "ALL"
            ),
            "Max3phaseSinglePackCurrent": GeckoByteStructAccessor(
                self.struct, "Max3phaseSinglePackCurrent", 167, "ALL"
            ),
            "Max1phaseDualPackCurrent": GeckoByteStructAccessor(
                self.struct, "Max1phaseDualPackCurrent", 168, "ALL"
            ),
            "Max2phaseDualPackCurrent": GeckoByteStructAccessor(
                self.struct, "Max2phaseDualPackCurrent", 169, "ALL"
            ),
            "Max3phaseDualPackCurrent": GeckoByteStructAccessor(
                self.struct, "Max3phaseDualPackCurrent", 170, "ALL"
            ),
            "RealPackType": GeckoEnumStructAccessor(
                self.struct,
                "RealPackType",
                171,
                None,
                ["", "", "", "", "", "", "", "", "", "", "inYT", "", "inYJ"],
                None,
                None,
                None,
            ),
            "InputCurrent": GeckoByteStructAccessor(
                self.struct, "InputCurrent", 53, "ALL"
            ),
            "MinimumInputCurrent": GeckoByteStructAccessor(
                self.struct, "MinimumInputCurrent", 122, "ALL"
            ),
            "InputMenu": GeckoEnumStructAccessor(
                self.struct,
                "InputMenu",
                74,
                None,
                ["Standard", "DualPack"],
                None,
                None,
                "ALL",
            ),
            "Out1Fuse": GeckoEnumStructAccessor(
                self.struct,
                "Out1Fuse",
                83,
                None,
                ["F1", "F2", "F3", "F21", "F22", "F23", "Line1", "Line2", "Line3"],
                None,
                None,
                "ALL",
            ),
            "Out2Fuse": GeckoEnumStructAccessor(
                self.struct,
                "Out2Fuse",
                84,
                None,
                ["F1", "F2", "F3", "F21", "F22", "F23", "Line1", "Line2", "Line3"],
                None,
                None,
                "ALL",
            ),
            "Out3Fuse": GeckoEnumStructAccessor(
                self.struct,
                "Out3Fuse",
                85,
                None,
                ["F1", "F2", "F3", "F21", "F22", "F23", "Line1", "Line2", "Line3"],
                None,
                None,
                "ALL",
            ),
            "Out4Fuse": GeckoEnumStructAccessor(
                self.struct,
                "Out4Fuse",
                86,
                None,
                ["F1", "F2", "F3", "F21", "F22", "F23", "Line1", "Line2", "Line3"],
                None,
                None,
                "ALL",
            ),
            "Out5Fuse": GeckoEnumStructAccessor(
                self.struct,
                "Out5Fuse",
                87,
                None,
                ["F1", "F2", "F3", "F21", "F22", "F23", "Line1", "Line2", "Line3"],
                None,
                None,
                "ALL",
            ),
            "Out6Fuse": GeckoEnumStructAccessor(
                self.struct,
                "Out6Fuse",
                88,
                None,
                ["F1", "F2", "F3", "F21", "F22", "F23", "Line1", "Line2", "Line3"],
                None,
                None,
                "ALL",
            ),
            "Out7Fuse": GeckoEnumStructAccessor(
                self.struct,
                "Out7Fuse",
                89,
                None,
                ["F1", "F2", "F3", "F21", "F22", "F23", "Line1", "Line2", "Line3"],
                None,
                None,
                "ALL",
            ),
            "Out8Fuse": GeckoEnumStructAccessor(
                self.struct,
                "Out8Fuse",
                90,
                None,
                ["F1", "F2", "F3", "F21", "F22", "F23", "Line1", "Line2", "Line3"],
                None,
                None,
                "ALL",
            ),
            "Out9Fuse": GeckoEnumStructAccessor(
                self.struct,
                "Out9Fuse",
                91,
                None,
                ["F1", "F2", "F3", "F21", "F22", "F23", "Line1", "Line2", "Line3"],
                None,
                None,
                "ALL",
            ),
            "Out10Fuse": GeckoEnumStructAccessor(
                self.struct,
                "Out10Fuse",
                92,
                None,
                ["F1", "F2", "F3", "F21", "F22", "F23", "Line1", "Line2", "Line3"],
                None,
                None,
                "ALL",
            ),
            "Out11Fuse": GeckoEnumStructAccessor(
                self.struct,
                "Out11Fuse",
                93,
                None,
                ["F1", "F2", "F3", "F21", "F22", "F23", "Line1", "Line2", "Line3"],
                None,
                None,
                "ALL",
            ),
            "Out12Fuse": GeckoEnumStructAccessor(
                self.struct,
                "Out12Fuse",
                94,
                None,
                ["F1", "F2", "F3", "F21", "F22", "F23", "Line1", "Line2", "Line3"],
                None,
                None,
                "ALL",
            ),
            "Direct1Fuse": GeckoEnumStructAccessor(
                self.struct,
                "Direct1Fuse",
                95,
                None,
                ["F1", "F2", "F3", "F21", "F22", "F23", "Line1", "Line2", "Line3"],
                None,
                None,
                "ALL",
            ),
            "Direct2Fuse": GeckoEnumStructAccessor(
                self.struct,
                "Direct2Fuse",
                96,
                None,
                ["F1", "F2", "F3", "F21", "F22", "F23", "Line1", "Line2", "Line3"],
                None,
                None,
                "ALL",
            ),
            "OutHtrFuse": GeckoEnumStructAccessor(
                self.struct,
                "OutHtrFuse",
                97,
                None,
                ["F1", "F2", "F3", "F21", "F22", "F23", "Line1", "Line2", "Line3"],
                None,
                None,
                "ALL",
            ),
            "F1Current": GeckoByteStructAccessor(self.struct, "F1Current", 98, "ALL"),
            "F2Current": GeckoByteStructAccessor(self.struct, "F2Current", 99, "ALL"),
            "F3Current": GeckoByteStructAccessor(self.struct, "F3Current", 100, "ALL"),
            "F21Current": GeckoByteStructAccessor(
                self.struct, "F21Current", 101, "ALL"
            ),
            "F22Current": GeckoByteStructAccessor(
                self.struct, "F22Current", 102, "ALL"
            ),
            "F23Current": GeckoByteStructAccessor(
                self.struct, "F23Current", 103, "ALL"
            ),
            "F1Line": GeckoEnumStructAccessor(
                self.struct,
                "F1Line",
                104,
                None,
                ["", "", "", "", "", "", "Line1", "Line2", "Line3"],
                None,
                None,
                "ALL",
            ),
            "F2Line": GeckoEnumStructAccessor(
                self.struct,
                "F2Line",
                105,
                None,
                ["", "", "", "", "", "", "Line1", "Line2", "Line3"],
                None,
                None,
                "ALL",
            ),
            "F3Line": GeckoEnumStructAccessor(
                self.struct,
                "F3Line",
                106,
                None,
                ["", "", "", "", "", "", "Line1", "Line2", "Line3"],
                None,
                None,
                "ALL",
            ),
            "F21Line": GeckoEnumStructAccessor(
                self.struct,
                "F21Line",
                107,
                None,
                ["", "", "", "", "", "", "Line1", "Line2", "Line3"],
                None,
                None,
                "ALL",
            ),
            "F22Line": GeckoEnumStructAccessor(
                self.struct,
                "F22Line",
                108,
                None,
                ["", "", "", "", "", "", "Line1", "Line2", "Line3"],
                None,
                None,
                "ALL",
            ),
            "F23Line": GeckoEnumStructAccessor(
                self.struct,
                "F23Line",
                109,
                None,
                ["", "", "", "", "", "", "Line1", "Line2", "Line3"],
                None,
                None,
                "ALL",
            ),
            "FiltFreq": GeckoByteStructAccessor(self.struct, "FiltFreq", 3, "ALL"),
            "FiltStart": GeckoTimeStructAccessor(self.struct, "FiltStart", 4, "ALL"),
            "FiltDur": GeckoTimeStructAccessor(self.struct, "FiltDur", 6, "ALL"),
            "FiltDur2": GeckoTimeStructAccessor(self.struct, "FiltDur2", 116, "ALL"),
            "EconStart": GeckoTimeStructAccessor(self.struct, "EconStart", 8, "ALL"),
            "EconDur": GeckoTimeStructAccessor(self.struct, "EconDur", 10, "ALL"),
            "EconProgAvailable": GeckoEnumStructAccessor(
                self.struct,
                "EconProgAvailable",
                71,
                None,
                ["NA", "STANDARD", "OUTSIDE_FILTER"],
                None,
                None,
                "ALL",
            ),
            "UDProgEcon": GeckoBoolStructAccessor(
                self.struct, "UDProgEcon", 82, 0, "ALL"
            ),
            "EconControlableManually": GeckoBoolStructAccessor(
                self.struct, "EconControlableManually", 72, 2, "ALL"
            ),
            "SoakOnCustomKey": GeckoBoolStructAccessor(
                self.struct, "SoakOnCustomKey", 72, 0, "ALL"
            ),
            "OffOnCustomKey": GeckoBoolStructAccessor(
                self.struct, "OffOnCustomKey", 72, 1, "ALL"
            ),
            "CleanupOnCustomKey": GeckoBoolStructAccessor(
                self.struct, "CleanupOnCustomKey", 72, 3, "ALL"
            ),
            "AuxOnCustomKey": GeckoBoolStructAccessor(
                self.struct, "AuxOnCustomKey", 72, 4, "ALL"
            ),
            "QuickOnOffCustomKey": GeckoBoolStructAccessor(
                self.struct, "QuickOnOffCustomKey", 72, 5, "ALL"
            ),
            "MultiKeyOption": GeckoEnumStructAccessor(
                self.struct,
                "MultiKeyOption",
                76,
                None,
                ["NoBlowerOnI2C", "BlowerOnI2C"],
                None,
                None,
                "ALL",
            ),
            "MasterSlave": GeckoEnumStructAccessor(
                self.struct,
                "MasterSlave",
                73,
                None,
                ["", "Master", "Slave"],
                None,
                None,
                "ALL",
            ),
            "SlaveConfig": GeckoByteStructAccessor(
                self.struct, "SlaveConfig", 75, "ALL"
            ),
            "ProgOutAccessory": GeckoEnumStructAccessor(
                self.struct,
                "ProgOutAccessory",
                160,
                None,
                [
                    "NA",
                    "",
                    "",
                    "",
                    "",
                    "",
                    "",
                    "",
                    "",
                    "",
                    "",
                    "",
                    "",
                    "L120",
                    "",
                    "",
                    "",
                    "",
                    "",
                    "",
                    "",
                    "",
                    "",
                    "ONZEN",
                    "",
                    "",
                    "",
                    "AUX",
                ],
                None,
                None,
                "ALL",
            ),
            "ProgOutStart": GeckoTimeStructAccessor(
                self.struct, "ProgOutStart", 128, "ALL"
            ),
            "ProgOutDur": GeckoTimeStructAccessor(
                self.struct, "ProgOutDur", 130, "ALL"
            ),
            "ProgOutFreq": GeckoByteStructAccessor(
                self.struct, "ProgOutFreq", 132, "ALL"
            ),
            "SilentMode": GeckoEnumStructAccessor(
                self.struct,
                "SilentMode",
                157,
                None,
                ["NA", "OFF", "ECONOMY", "SLEEP", "NIGHT"],
                None,
                None,
                "ALL",
            ),
            "SilentDuration": GeckoTimeStructAccessor(
                self.struct, "SilentDuration", 158, "ALL"
            ),
            "SuspendSilentToRegulate": GeckoEnumStructAccessor(
                self.struct,
                "SuspendSilentToRegulate",
                119,
                3,
                ["NOT_ALLOWED", "ALLOWED"],
                None,
                2,
                "ALL",
            ),
            "TimeFormat": GeckoEnumStructAccessor(
                self.struct,
                "TimeFormat",
                34,
                None,
                ["NA", "AmPm", "24h"],
                None,
                None,
                "ALL",
            ),
            "AmbiantOHTrigADC": GeckoWordStructAccessor(
                self.struct, "AmbiantOHTrigADC", 64, "ALL"
            ),
            "Pump1UserAccess": GeckoEnumStructAccessor(
                self.struct,
                "Pump1UserAccess",
                81,
                1,
                ["BothSpeeds", "HighSpeedOnly"],
                None,
                2,
                "ALL",
            ),
            "SelfCleanMsg": GeckoBoolStructAccessor(
                self.struct, "SelfCleanMsg", 81, 4, "ALL"
            ),
            "BlowerKeyOption": GeckoEnumStructAccessor(
                self.struct,
                "BlowerKeyOption",
                81,
                5,
                ["FreePumpKey", "LastPumpKey"],
                None,
                2,
                "ALL",
            ),
            "HeaterSoftStart": GeckoBoolStructAccessor(
                self.struct, "HeaterSoftStart", 81, 6, "ALL"
            ),
            "HeaterSoftStop": GeckoBoolStructAccessor(
                self.struct, "HeaterSoftStop", 81, 7, "ALL"
            ),
            "KeypadTherapySupport": GeckoBoolStructAccessor(
                self.struct, "KeypadTherapySupport", 110, 0, "ALL"
            ),
            "CustomKeyEnabled": GeckoBoolStructAccessor(
                self.struct, "CustomKeyEnabled", 110, 1, "ALL"
            ),
            "ConfigChange": GeckoEnumStructAccessor(
                self.struct,
                "ConfigChange",
                110,
                2,
                ["NO_RESTRICTION", "PASSWORD_PROTECTED"],
                None,
                2,
                "ALL",
            ),
            "BreakerChange": GeckoEnumStructAccessor(
                self.struct,
                "BreakerChange",
                110,
                3,
                ["NO_RESTRICTION", "PASSWORD_PROTECTED"],
                None,
                2,
                "ALL",
            ),
            "KeypadBacklightColor": GeckoEnumStructAccessor(
                self.struct,
                "KeypadBacklightColor",
                110,
                4,
                ["OFF", "RED", "GREEN", "YELLOW", "BLUE", "MAGENTA", "CYAN", "WHITE"],
                None,
                8,
                "ALL",
            ),
            "KeypadBacklightEdit": GeckoEnumStructAccessor(
                self.struct,
                "KeypadBacklightEdit",
                110,
                7,
                ["Disable", "Enable"],
                None,
                2,
                "ALL",
            ),
            "ModeKeyAsInvertDisplayKey": GeckoBoolStructAccessor(
                self.struct, "ModeKeyAsInvertDisplayKey", 112, 1, "ALL"
            ),
            "InfoMsgConfig": GeckoEnumStructAccessor(
                self.struct,
                "InfoMsgConfig",
                112,
                2,
                ["HIDE_DETAILED_MSG", "SHOW_ALL_MSG", "", ""],
                None,
                4,
                "ALL",
            ),
            "LowerSetpointMenu": GeckoBoolStructAccessor(
                self.struct, "LowerSetpointMenu", 112, 4, "ALL"
            ),
            "SinglePumpKey": GeckoEnumStructAccessor(
                self.struct,
                "SinglePumpKey",
                112,
                6,
                ["NA", "ForSpeeds", "ForZones", ""],
                None,
                4,
                "ALL",
            ),
            "KeypadOptions4": GeckoByteStructAccessor(
                self.struct, "KeypadOptions4", 113, "ALL"
            ),
            "DealerLockSupport": GeckoBoolStructAccessor(
                self.struct, "DealerLockSupport", 119, 0, "ALL"
            ),
            "LockEnabled": GeckoByteStructAccessor(
                self.struct, "LockEnabled", 121, "ALL"
            ),
            "VSPType": GeckoEnumStructAccessor(
                self.struct,
                "VSPType",
                181,
                None,
                ["NoVsp", "Hydropool", "Gecko_VMS_OnModbus"],
                None,
                None,
                "ALL",
            ),
            "VSPHeatingSpeed": GeckoByteStructAccessor(
                self.struct, "VSPHeatingSpeed", 182, "ALL"
            ),
            "VSPFilteringSpeed": GeckoByteStructAccessor(
                self.struct, "VSPFilteringSpeed", 183, "ALL"
            ),
            "AssignVSP1": GeckoEnumStructAccessor(
                self.struct,
                "AssignVSP1",
                173,
                None,
                ["NA", "P1", "P2", "P3", "P4", "P5"],
                None,
                None,
                "ALL",
            ),
            "AssignVSP2": GeckoEnumStructAccessor(
                self.struct,
                "AssignVSP2",
                174,
                None,
                ["NA", "P1", "P2", "P3", "P4", "P5"],
                None,
                None,
                "ALL",
            ),
            "AssignVSP3": GeckoEnumStructAccessor(
                self.struct,
                "AssignVSP3",
                175,
                None,
                ["NA", "P1", "P2", "P3", "P4", "P5"],
                None,
                None,
                "ALL",
            ),
            "AssignVSP4": GeckoEnumStructAccessor(
                self.struct,
                "AssignVSP4",
                176,
                None,
                ["NA", "P1", "P2", "P3", "P4", "P5"],
                None,
                None,
                "ALL",
            ),
            "AssignVSP5": GeckoEnumStructAccessor(
                self.struct,
                "AssignVSP5",
                177,
                None,
                ["NA", "P1", "P2", "P3", "P4", "P5"],
                None,
                None,
                "ALL",
            ),
            "ExerciseType": GeckoEnumStructAccessor(
                self.struct,
                "ExerciseType",
                133,
                None,
                ["NO_EXERCISE", "SWIM_EXERCISE"],
                None,
                None,
                "ALL",
            ),
            "ExerciseControl": GeckoEnumStructAccessor(
                self.struct,
                "ExerciseControl",
                134,
                None,
                ["USE_STANDARD_PUMPS", "USE_VARIABLE_SPEED_PUMPS"],
                None,
                None,
                "ALL",
            ),
            "ExercisePumpsUsed": GeckoEnumStructAccessor(
                self.struct,
                "ExercisePumpsUsed",
                135,
                0,
                [
                    "NO_PUMP",
                    "ALL_PUMPS",
                    "P2_TO_P5",
                    "P3_TO_P5",
                    "P4_AND_P5",
                    "P5_ONLY",
                    "",
                    "",
                ],
                None,
                8,
                "ALL",
            ),
            "ExerciseDedicatedPumps": GeckoBoolStructAccessor(
                self.struct, "ExerciseDedicatedPumps", 135, 7, "ALL"
            ),
            "ExerciseBuoyancyPump": GeckoEnumStructAccessor(
                self.struct,
                "ExerciseBuoyancyPump",
                178,
                None,
                ["NA", "P1", "P2", "P3", "P4", "P5"],
                None,
                None,
                "ALL",
            ),
            "ExerciseNumberOfIntensity": GeckoByteStructAccessor(
                self.struct, "ExerciseNumberOfIntensity", 136, "ALL"
            ),
            "ExcercisePumpIntensity1": GeckoWordStructAccessor(
                self.struct, "ExcercisePumpIntensity1", 137, "ALL"
            ),
            "ExcercisePumpIntensity2": GeckoWordStructAccessor(
                self.struct, "ExcercisePumpIntensity2", 139, "ALL"
            ),
            "ExcercisePumpIntensity3": GeckoWordStructAccessor(
                self.struct, "ExcercisePumpIntensity3", 141, "ALL"
            ),
            "ExcercisePumpIntensity4": GeckoWordStructAccessor(
                self.struct, "ExcercisePumpIntensity4", 143, "ALL"
            ),
            "ExcercisePumpIntensity5": GeckoWordStructAccessor(
                self.struct, "ExcercisePumpIntensity5", 145, "ALL"
            ),
            "ExcercisePumpIntensity6": GeckoWordStructAccessor(
                self.struct, "ExcercisePumpIntensity6", 147, "ALL"
            ),
            "ExcercisePumpIntensity7": GeckoWordStructAccessor(
                self.struct, "ExcercisePumpIntensity7", 149, "ALL"
            ),
            "ExcercisePumpIntensity8": GeckoWordStructAccessor(
                self.struct, "ExcercisePumpIntensity8", 151, "ALL"
            ),
            "ExcercisePumpIntensity9": GeckoWordStructAccessor(
                self.struct, "ExcercisePumpIntensity9", 153, "ALL"
            ),
            "ExcercisePumpIntensity10": GeckoWordStructAccessor(
                self.struct, "ExcercisePumpIntensity10", 155, "ALL"
            ),
            "Zone1Led": GeckoEnumStructAccessor(
                self.struct, "Zone1Led", 123, 0, ["RGB", "WHITE"], None, 2, None
            ),
            "Zone1Type": GeckoEnumStructAccessor(
                self.struct, "Zone1Type", 123, 1, ["NORMAL", "STATUS"], None, 2, None
            ),
            "Zone2Led": GeckoEnumStructAccessor(
                self.struct, "Zone2Led", 124, 0, ["RGB", "WHITE"], None, 2, None
            ),
            "Zone2Type": GeckoEnumStructAccessor(
                self.struct, "Zone2Type", 124, 1, ["NORMAL", "STATUS"], None, 2, None
            ),
            "Zone3Led": GeckoEnumStructAccessor(
                self.struct, "Zone3Led", 125, 0, ["RGB", "WHITE"], None, 2, None
            ),
            "Zone3Type": GeckoEnumStructAccessor(
                self.struct, "Zone3Type", 125, 1, ["NORMAL", "STATUS"], None, 2, None
            ),
            "Zone4Led": GeckoEnumStructAccessor(
                self.struct, "Zone4Led", 126, 0, ["RGB", "WHITE"], None, 2, None
            ),
            "Zone4Type": GeckoEnumStructAccessor(
                self.struct, "Zone4Type", 126, 1, ["NORMAL", "STATUS"], None, 2, None
            ),
            "NumberOfZones": GeckoEnumStructAccessor(
                self.struct,
                "NumberOfZones",
                127,
                0,
                ["", "1", "2", "3", "4"],
                None,
                8,
                None,
            ),
            "StatusLightNormalColor": GeckoEnumStructAccessor(
                self.struct,
                "StatusLightNormalColor",
                127,
                3,
                ["BLUE", "WHITE", "GREEN", "PURPLE", "YELLOW", "CYAN", "ORANGE", "OFF"],
                None,
                8,
                "ALL",
            ),
            "MappingEnable": GeckoBoolStructAccessor(
                self.struct, "MappingEnable", 127, 7, None
            ),
            "MapZone1ToOut1": GeckoBoolStructAccessor(
                self.struct, "MapZone1ToOut1", 123, 4, None
            ),
            "MapZone1ToOut2": GeckoBoolStructAccessor(
                self.struct, "MapZone1ToOut2", 123, 5, None
            ),
            "MapZone1ToOut3": GeckoBoolStructAccessor(
                self.struct, "MapZone1ToOut3", 123, 6, None
            ),
            "MapZone1ToOut4": GeckoBoolStructAccessor(
                self.struct, "MapZone1ToOut4", 123, 7, None
            ),
            "MapZone2ToOut1": GeckoBoolStructAccessor(
                self.struct, "MapZone2ToOut1", 124, 4, None
            ),
            "MapZone2ToOut2": GeckoBoolStructAccessor(
                self.struct, "MapZone2ToOut2", 124, 5, None
            ),
            "MapZone2ToOut3": GeckoBoolStructAccessor(
                self.struct, "MapZone2ToOut3", 124, 6, None
            ),
            "MapZone2ToOut4": GeckoBoolStructAccessor(
                self.struct, "MapZone2ToOut4", 124, 7, None
            ),
            "MapZone3ToOut1": GeckoBoolStructAccessor(
                self.struct, "MapZone3ToOut1", 125, 4, None
            ),
            "MapZone3ToOut2": GeckoBoolStructAccessor(
                self.struct, "MapZone3ToOut2", 125, 5, None
            ),
            "MapZone3ToOut3": GeckoBoolStructAccessor(
                self.struct, "MapZone3ToOut3", 125, 6, None
            ),
            "MapZone3ToOut4": GeckoBoolStructAccessor(
                self.struct, "MapZone3ToOut4", 125, 7, None
            ),
            "MapZone4ToOut1": GeckoBoolStructAccessor(
                self.struct, "MapZone4ToOut1", 126, 4, None
            ),
            "MapZone4ToOut2": GeckoBoolStructAccessor(
                self.struct, "MapZone4ToOut2", 126, 5, None
            ),
            "MapZone4ToOut3": GeckoBoolStructAccessor(
                self.struct, "MapZone4ToOut3", 126, 6, None
            ),
            "MapZone4ToOut4": GeckoBoolStructAccessor(
                self.struct, "MapZone4ToOut4", 126, 7, None
            ),
            "InMixZoneToFollow": GeckoEnumStructAccessor(
                self.struct,
                "InMixZoneToFollow",
                172,
                None,
                ["ZONE1", "ZONE2", "ZONE3", "ZONE4", "AS_12V_SUPPLY"],
                None,
                None,
                None,
            ),
            "StatusLightRemindersColor": GeckoEnumStructAccessor(
                self.struct,
                "StatusLightRemindersColor",
                179,
                4,
                [
                    "SAME_AS_NORMAL",
                    "WHITE",
                    "GREEN",
                    "PURPLE",
                    "YELLOW",
                    "CYAN",
                    "ORANGE",
                    "OFF",
                ],
                None,
                8,
                "ALL",
            ),
            "RemindersNotBlinkingLED": GeckoBoolStructAccessor(
                self.struct, "RemindersNotBlinkingLED", 179, 7, None
            ),
            "StatusLightFading": GeckoBoolStructAccessor(
                self.struct, "StatusLightFading", 184, 0, None
            ),
        }



#!/usr/bin/python3
"""
    GeckoLogStruct - A class to manage the LogStruct for 'InXM v2'
"""

from . import (
    GeckoByteStructAccessor,
    GeckoWordStructAccessor,
    GeckoTimeStructAccessor,
    GeckoBoolStructAccessor,
    GeckoEnumStructAccessor,
    GeckoTempStructAccessor,
)


class GeckoLogStruct:
    def __init__(self, struct_):
        self.struct = struct_

    @property
    def version(self):
        return 2

    @property
    def begin(self):
        return 256

    @property
    def end(self):
        return 511

    @property
    def all_device_keys(self):
        return [
            "P1",
            "P2",
            "P3",
            "P4",
            "P5",
            "BLOWER",
            "CP",
            "OZONE",
            "L120",
            "FAN",
            "RH_DutyCycle",
            "VSP1_AvantPlanSpeed",
            "VSP1_MinimumSpeed",
            "VSP3_AvantPlanSpeed",
            "VSP3_MinimumSpeed",
            "LI",
        ]

    @property
    def user_demand_keys(self):
        return []

    @property
    def error_keys(self):
        return [
            "RH_DutyCycleErr",
            "AmbiantOHLevel2",
            "RH_CommErr",
            "ThermistanceErr",
            "rHId",
            "Fuse3Err",
            "SupplyErr",
            "Fuse2Err",
            "RegOverHeatFlag",
            "KinPumpOff",
            "ScanErr",
            "ContactorCoilFailErr",
            "OHCondition",
            "RelayStuck",
            "LearnDetectionErr",
            "NoHeaterCurrentErr",
            "AmbiantOHLevel1",
            "RH_RegProbeErr",
            "Fuse1Err",
        ]

    @property
    def accessors(self):
        return {
            "Hours": GeckoByteStructAccessor(self.struct, "Hours", 256, "ALL"),
            "BrMenu": GeckoBoolStructAccessor(self.struct, "BrMenu", 279, 6, None),
            "HeaterPhase": GeckoByteStructAccessor(
                self.struct, "HeaterPhase", 300, None
            ),
            "LearningCurrentOUT1A": GeckoWordStructAccessor(
                self.struct, "LearningCurrentOUT1A", 280, None
            ),
            "LearningCurrentOUT1B": GeckoWordStructAccessor(
                self.struct, "LearningCurrentOUT1B", 282, None
            ),
            "LearningCurrentOUT2A": GeckoWordStructAccessor(
                self.struct, "LearningCurrentOUT2A", 284, None
            ),
            "LearningCurrentOUT2B": GeckoWordStructAccessor(
                self.struct, "LearningCurrentOUT2B", 286, None
            ),
            "LearningCurrentOUT3A": GeckoWordStructAccessor(
                self.struct, "LearningCurrentOUT3A", 288, None
            ),
            "LearningCurrentOUT4A": GeckoWordStructAccessor(
                self.struct, "LearningCurrentOUT4A", 290, None
            ),
            "LearningCurrentOUT5A": GeckoWordStructAccessor(
                self.struct, "LearningCurrentOUT5A", 292, None
            ),
            "LearningCurrentOUT6A": GeckoWordStructAccessor(
                self.struct, "LearningCurrentOUT6A", 294, None
            ),
            "LearningCurrentOUT7A": GeckoWordStructAccessor(
                self.struct, "LearningCurrentOUT7A", 296, None
            ),
            "CurrentHeater": GeckoWordStructAccessor(
                self.struct, "CurrentHeater", 298, "ALL"
            ),
            "UD_P1": GeckoEnumStructAccessor(
                self.struct, "UD_P1", 275, 14, ["OFF", "LOW", "HIGH"], 2, 4, "ALL"
            ),
            "UD_P2": GeckoEnumStructAccessor(
                self.struct, "UD_P2", 275, 12, ["OFF", "LOW", "HIGH"], 2, 4, "Gecko"
            ),
            "UD_P3": GeckoEnumStructAccessor(
                self.struct, "UD_P3", 275, 11, ["OFF", "HIGH"], 2, 2, "RD"
            ),
            "UD_P4": GeckoEnumStructAccessor(
                self.struct, "UD_P4", 275, 10, ["OFF", "HIGH"], 2, 2, "ALL"
            ),
            "UD_P5": GeckoEnumStructAccessor(
                self.struct, "UD_P5", 275, 9, ["OFF", "HIGH"], 2, 2, "ALL"
            ),
            "UD_BLOWER": GeckoEnumStructAccessor(
                self.struct, "UD_BLOWER", 275, 8, ["OFF", "ON"], 2, 2, "ALL"
            ),
            "UD_AUX1": GeckoEnumStructAccessor(
                self.struct, "UD_AUX1", 275, 5, ["OFF", "ON"], 2, 2, "ALL"
            ),
            "UD_AUX2": GeckoEnumStructAccessor(
                self.struct, "UD_AUX2", 275, 4, ["OFF", "ON"], 2, 2, "ALL"
            ),
            "UD_FB_LT": GeckoEnumStructAccessor(
                self.struct,
                "UD_FB_LT",
                275,
                2,
                ["OFF", "FIXCOLOR", "SCAN"],
                2,
                4,
                "ALL",
            ),
            "UD_LIGHT120": GeckoEnumStructAccessor(
                self.struct, "UD_LIGHT120", 275, 1, ["OFF", "ON"], 2, 2, "ALL"
            ),
            "UD_QUIET_PAUSE": GeckoEnumStructAccessor(
                self.struct, "UD_QUIET_PAUSE", 275, 0, [" ", "QUIET"], 2, 2, "ALL"
            ),
            "VSP1_UDSpeed": GeckoByteStructAccessor(
                self.struct, "VSP1_UDSpeed", 304, "ALL"
            ),
            "VSP3_UDSpeed": GeckoByteStructAccessor(
                self.struct, "VSP3_UDSpeed", 305, "ALL"
            ),
            "LightIntensity": GeckoByteStructAccessor(
                self.struct, "LightIntensity", 308, "ALL"
            ),
            "P1": GeckoEnumStructAccessor(
                self.struct, "P1", 316, 0, ["OFF", "HIGH", "LOW"], None, 4, None
            ),
            "P2": GeckoEnumStructAccessor(
                self.struct, "P2", 316, 2, ["OFF", "HIGH", "LOW"], None, 4, None
            ),
            "P3": GeckoEnumStructAccessor(
                self.struct, "P3", 316, 4, ["OFF", "HIGH", "LOW"], None, 4, None
            ),
            "P4": GeckoEnumStructAccessor(
                self.struct, "P4", 316, 6, ["OFF", "HIGH", "LOW"], None, 4, None
            ),
            "P5": GeckoEnumStructAccessor(
                self.struct, "P5", 315, 0, ["OFF", "HIGH"], None, 2, None
            ),
            "BLOWER": GeckoEnumStructAccessor(
                self.struct, "BLOWER", 315, 1, ["OFF", "HIGH"], None, 2, None
            ),
            "CP": GeckoEnumStructAccessor(
                self.struct, "CP", 315, 2, ["OFF", "ON"], None, 2, None
            ),
            "OZONE": GeckoEnumStructAccessor(
                self.struct, "OZONE", 315, 3, ["OFF", "ON"], None, 2, None
            ),
            "L120": GeckoEnumStructAccessor(
                self.struct, "L120", 315, 4, ["OFF", "ON"], None, 2, None
            ),
            "FAN": GeckoEnumStructAccessor(
                self.struct, "FAN", 315, 7, ["OFF", "ON"], None, 2, None
            ),
            "RH_DutyCycle": GeckoByteStructAccessor(
                self.struct, "RH_DutyCycle", 274, None
            ),
            "VSP1_AvantPlanSpeed": GeckoByteStructAccessor(
                self.struct, "VSP1_AvantPlanSpeed", 302, None
            ),
            "VSP1_MinimumSpeed": GeckoByteStructAccessor(
                self.struct, "VSP1_MinimumSpeed", 306, None
            ),
            "VSP3_AvantPlanSpeed": GeckoByteStructAccessor(
                self.struct, "VSP3_AvantPlanSpeed", 303, None
            ),
            "VSP3_MinimumSpeed": GeckoByteStructAccessor(
                self.struct, "VSP3_MinimumSpeed", 307, None
            ),
            "RemoteFiltAction": GeckoEnumStructAccessor(
                self.struct,
                "RemoteFiltAction",
                309,
                None,
                ["USE_INTERNAL", "IDLE", "STOP", "START", "NEW", "FILTER"],
                None,
                6,
                "ALL",
            ),
            "RemoteFiltDur": GeckoTimeStructAccessor(
                self.struct, "RemoteFiltDur", 310, "ALL"
            ),
            "RemoteOnzenAction": GeckoEnumStructAccessor(
                self.struct,
                "RemoteOnzenAction",
                312,
                None,
                ["USE_INTERNAL", "IDLE", "STOP", "START", "NEW", "FILTER"],
                None,
                6,
                "ALL",
            ),
            "RemoteOnzenDur": GeckoTimeStructAccessor(
                self.struct, "RemoteOnzenDur", 313, "ALL"
            ),
            "FiltPurge": GeckoBoolStructAccessor(
                self.struct, "FiltPurge", 269, 7, None
            ),
            "FiltClean": GeckoBoolStructAccessor(
                self.struct, "FiltClean", 269, 5, None
            ),
            "FiltCleanSuspended": GeckoBoolStructAccessor(
                self.struct, "FiltCleanSuspended", 269, 4, None
            ),
            "FiltPurgeSuspendUD": GeckoBoolStructAccessor(
                self.struct, "FiltPurgeSuspendUD", 279, 2, None
            ),
            "FiltCleanSuspendUD": GeckoBoolStructAccessor(
                self.struct, "FiltCleanSuspendUD", 279, 3, None
            ),
            "FilterOT": GeckoBoolStructAccessor(self.struct, "FilterOT", 279, 0, None),
            "CPOT": GeckoBoolStructAccessor(self.struct, "CPOT", 279, 1, None),
            "SWM_Active": GeckoByteStructAccessor(self.struct, "SWM_Active", 267, None),
            "SWM_Risk": GeckoEnumStructAccessor(
                self.struct,
                "SWM_Risk",
                268,
                None,
                ["NO", "LOW", "MEDIUM", "HIGH", "EXTREME"],
                None,
                8,
                None,
            ),
            "SWMPurge": GeckoBoolStructAccessor(self.struct, "SWMPurge", 269, 3, None),
            "SWMPurge_Susp": GeckoBoolStructAccessor(
                self.struct, "SWMPurge_Susp", 269, 2, None
            ),
            "DisplayedTemp_G": GeckoTempStructAccessor(
                self.struct, "DisplayedTemp_G", 265, None
            ),
            "PumpRun": GeckoBoolStructAccessor(self.struct, "PumpRun", 269, 1, None),
            "CheckFlow": GeckoBoolStructAccessor(
                self.struct, "CheckFlow", 269, 0, None
            ),
            "Heating": GeckoBoolStructAccessor(self.struct, "Heating", 270, 7, None),
            "CoolingDown": GeckoBoolStructAccessor(
                self.struct, "CoolingDown", 270, 6, None
            ),
            "KineticPurgeActive": GeckoBoolStructAccessor(
                self.struct, "KineticPurgeActive", 270, 5, None
            ),
            "EconomyActive": GeckoBoolStructAccessor(
                self.struct, "EconomyActive", 271, 0, None
            ),
            "ProgEconomyActive": GeckoBoolStructAccessor(
                self.struct, "ProgEconomyActive", 271, 2, None
            ),
            "ExtProbeActive": GeckoBoolStructAccessor(
                self.struct, "ExtProbeActive", 301, 0, None
            ),
            "ThermistanceErr": GeckoBoolStructAccessor(
                self.struct, "ThermistanceErr", 277, 5, None
            ),
            "ContactorCoilFailErr": GeckoBoolStructAccessor(
                self.struct, "ContactorCoilFailErr", 277, 4, None
            ),
            "NoHeaterCurrentErr": GeckoBoolStructAccessor(
                self.struct, "NoHeaterCurrentErr", 277, 3, None
            ),
            "LearnDetectionErr": GeckoBoolStructAccessor(
                self.struct, "LearnDetectionErr", 277, 2, None
            ),
            "Fuse3Err": GeckoBoolStructAccessor(self.struct, "Fuse3Err", 277, 1, None),
            "Fuse2Err": GeckoBoolStructAccessor(self.struct, "Fuse2Err", 277, 0, None),
            "Fuse1Err": GeckoBoolStructAccessor(self.struct, "Fuse1Err", 278, 7, None),
            "SupplyErr": GeckoBoolStructAccessor(
                self.struct, "SupplyErr", 278, 6, None
            ),
            "ScanErr": GeckoBoolStructAccessor(self.struct, "ScanErr", 278, 5, None),
            "RegOverHeatFlag": GeckoBoolStructAccessor(
                self.struct, "RegOverHeatFlag", 278, 4, None
            ),
            "AmbiantOHLevel2": GeckoBoolStructAccessor(
                self.struct, "AmbiantOHLevel2", 278, 3, None
            ),
            "AmbiantOHLevel1": GeckoBoolStructAccessor(
                self.struct, "AmbiantOHLevel1", 278, 2, None
            ),
            "RelayStuck": GeckoBoolStructAccessor(
                self.struct, "RelayStuck", 278, 1, None
            ),
            "rHId": GeckoBoolStructAccessor(self.struct, "rHId", 278, 0, None),
            "KinPumpOff": GeckoBoolStructAccessor(
                self.struct, "KinPumpOff", 279, 4, None
            ),
            "OHCondition": GeckoBoolStructAccessor(
                self.struct, "OHCondition", 279, 5, None
            ),
            "RH_WaterTemp": GeckoTempStructAccessor(
                self.struct, "RH_WaterTemp", 261, None
            ),
            "RH_TriacTemp": GeckoTempStructAccessor(
                self.struct, "RH_TriacTemp", 263, None
            ),
            "RH_CommErr": GeckoBoolStructAccessor(
                self.struct, "RH_CommErr", 257, 7, None
            ),
            "RH_NotEnoughHeat": GeckoBoolStructAccessor(
                self.struct, "RH_NotEnoughHeat", 257, 3, None
            ),
            "RH_RegSlope": GeckoBoolStructAccessor(
                self.struct, "RH_RegSlope", 257, 2, None
            ),
            "RH_LowFlo": GeckoBoolStructAccessor(
                self.struct, "RH_LowFlo", 257, 0, None
            ),
            "RH_DutyCycleErr": GeckoBoolStructAccessor(
                self.struct, "RH_DutyCycleErr", 258, 7, None
            ),
            "RH_NoFloRetry": GeckoBoolStructAccessor(
                self.struct, "RH_NoFloRetry", 258, 6, None
            ),
            "RH_NoFloTemp": GeckoBoolStructAccessor(
                self.struct, "RH_NoFloTemp", 258, 5, None
            ),
            "RH_NoFloXTries": GeckoBoolStructAccessor(
                self.struct, "RH_NoFloXTries", 258, 4, None
            ),
            "RH_HR_HL": GeckoBoolStructAccessor(self.struct, "RH_HR_HL", 258, 3, None),
            "RH_HW_HL": GeckoBoolStructAccessor(self.struct, "RH_HW_HL", 258, 2, None),
            "RH_RegProbeErr": GeckoBoolStructAccessor(
                self.struct, "RH_RegProbeErr", 258, 1, None
            ),
            "RH_RegOverHeat": GeckoBoolStructAccessor(
                self.struct, "RH_RegOverHeat", 258, 0, None
            ),
            "RH_HrTriacPr": GeckoBoolStructAccessor(
                self.struct, "RH_HrTriacPr", 259, 7, None
            ),
            "RH_HrTriacOH": GeckoBoolStructAccessor(
                self.struct, "RH_HrTriacOH", 259, 6, None
            ),
            "RH_SWTriacOH": GeckoBoolStructAccessor(
                self.struct, "RH_SWTriacOH", 259, 5, None
            ),
            "RH_HwTriacOH": GeckoBoolStructAccessor(
                self.struct, "RH_HwTriacOH", 259, 4, None
            ),
            "RH_HrKin": GeckoBoolStructAccessor(self.struct, "RH_HrKin", 259, 3, None),
            "RH_HrKinNoFlo": GeckoBoolStructAccessor(
                self.struct, "RH_HrKinNoFlo", 259, 2, None
            ),
            "RH_SwKinTemp": GeckoBoolStructAccessor(
                self.struct, "RH_SwKinTemp", 259, 1, None
            ),
            "RH_HWKinetic": GeckoBoolStructAccessor(
                self.struct, "RH_HWKinetic", 259, 0, None
            ),
            "RH_HeaterDisable": GeckoBoolStructAccessor(
                self.struct, "RH_HeaterDisable", 260, 4, None
            ),
            "RH_FlowDetected": GeckoBoolStructAccessor(
                self.struct, "RH_FlowDetected", 260, 3, None
            ),
            "RH_FlowCheck": GeckoBoolStructAccessor(
                self.struct, "RH_FlowCheck", 260, 2, None
            ),
            "RH_HeaterSuspended": GeckoBoolStructAccessor(
                self.struct, "RH_HeaterSuspended", 260, 1, None
            ),
            "RH_FloCheckReady": GeckoBoolStructAccessor(
                self.struct, "RH_FloCheckReady", 260, 0, None
            ),
        }



#!/usr/bin/python3
"""
    GeckoConfigStruct - A class to manage the ConfigStruct for 'InXE v9'
"""

from . import (
    GeckoByteStructAccessor,
    GeckoWordStructAccessor,
    GeckoTimeStructAccessor,
    GeckoBoolStructAccessor,
    GeckoEnumStructAccessor,
    GeckoTempStructAccessor,
)


class GeckoConfigStruct:
    def __init__(self, struct_):
        self.struct = struct_

    @property
    def version(self):
        return 9

    @property
    def output_keys(self):
        return ["Out1", "Out2", "Out3", "Out4", "Out5", "OutHtr"]

    @property
    def accessors(self):
        return {
            "ConfigNumber": GeckoByteStructAccessor(
                self.struct, "ConfigNumber", 0, "ALL"
            ),
            "Out1": GeckoEnumStructAccessor(
                self.struct,
                "Out1",
                9,
                None,
                [
                    "NA",
                    "P1H",
                    "P1L",
                    "P2H",
                    "P2L",
                    "P3H",
                    "P3L",
                    "P4H",
                    "P4L",
                    "P5",
                    "BLO",
                    "CP",
                    "O3",
                    "L120",
                    "",
                    "",
                    "",
                    "",
                    "",
                    "FullOn",
                ],
                None,
                None,
                "ALL",
            ),
            "Out2": GeckoEnumStructAccessor(
                self.struct,
                "Out2",
                10,
                None,
                [
                    "NA",
                    "P1H",
                    "P1L",
                    "P2H",
                    "P2L",
                    "P3H",
                    "P3L",
                    "P4H",
                    "P4L",
                    "P5",
                    "BLO",
                    "CP",
                    "O3",
                    "L120",
                    "",
                    "",
                    "",
                    "",
                    "",
                    "FullOn",
                ],
                None,
                None,
                "ALL",
            ),
            "Out3": GeckoEnumStructAccessor(
                self.struct,
                "Out3",
                11,
                None,
                [
                    "NA",
                    "P1H",
                    "P1L",
                    "P2H",
                    "P2L",
                    "P3H",
                    "P3L",
                    "P4H",
                    "P4L",
                    "P5",
                    "BLO",
                    "CP",
                    "O3",
                    "L120",
                    "",
                    "",
                    "",
                    "",
                    "",
                    "FullOn",
                ],
                None,
                None,
                "ALL",
            ),
            "Out4": GeckoEnumStructAccessor(
                self.struct,
                "Out4",
                12,
                None,
                [
                    "NA",
                    "P1H",
                    "P1L",
                    "P2H",
                    "P2L",
                    "P3H",
                    "P3L",
                    "P4H",
                    "P4L",
                    "P5",
                    "BLO",
                    "CP",
                    "O3",
                    "L120",
                    "",
                    "",
                    "",
                    "",
                    "",
                    "FullOn",
                ],
                None,
                None,
                "ALL",
            ),
            "Out5": GeckoEnumStructAccessor(
                self.struct,
                "Out5",
                13,
                None,
                [
                    "NA",
                    "P1H",
                    "P1L",
                    "P2H",
                    "P2L",
                    "P3H",
                    "P3L",
                    "P4H",
                    "P4L",
                    "P5",
                    "BLO",
                    "CP",
                    "O3",
                    "L120",
                    "",
                    "",
                    "",
                    "",
                    "",
                    "FullOn",
                ],
                None,
                None,
                "ALL",
            ),
            "OutHtr": GeckoEnumStructAccessor(
                self.struct,
                "OutHtr",
                14,
                None,
                ["NA", "", "", "", "", "", "", "", "", "", "", "", "", "", "HTR"],
                None,
                None,
                "ALL",
            ),
            "Out1Cur": GeckoByteStructAccessor(self.struct, "Out1Cur", 23, "ALL"),
            "Out2Cur": GeckoByteStructAccessor(self.struct, "Out2Cur", 24, "ALL"),
            "Out3Cur": GeckoByteStructAccessor(self.struct, "Out3Cur", 25, "ALL"),
            "Out4Cur": GeckoByteStructAccessor(self.struct, "Out4Cur", 26, "ALL"),
            "Out5Cur": GeckoByteStructAccessor(self.struct, "Out5Cur", 27, "ALL"),
            "OutHtRCur": GeckoByteStructAccessor(self.struct, "OutHtRCur", 28, "ALL"),
            "DirectCur": GeckoByteStructAccessor(self.struct, "DirectCur", 29, "ALL"),
            "PumpTimeOut": GeckoByteStructAccessor(
                self.struct, "PumpTimeOut", 33, "ALL"
            ),
            "LightTimeOut": GeckoByteStructAccessor(
                self.struct, "LightTimeOut", 34, "ALL"
            ),
            "L120TimeOut": GeckoByteStructAccessor(
                self.struct, "L120TimeOut", 35, "ALL"
            ),
            "FiltInterface": GeckoEnumStructAccessor(
                self.struct,
                "FiltInterface",
                20,
                None,
                ["PurgeOnly", "FiltCP", "FiltP1"],
                None,
                None,
                "ALL",
            ),
            "CPAlwaysON": GeckoBoolStructAccessor(
                self.struct, "CPAlwaysON", 15, 0, "ALL"
            ),
            "OTActSetpointG": GeckoTempStructAccessor(
                self.struct, "OTActSetpointG", 36, "ALL"
            ),
            "OTTriggerG": GeckoByteStructAccessor(self.struct, "OTTriggerG", 38, "ALL"),
            "CPOTMaxOnTime": GeckoWordStructAccessor(
                self.struct, "CPOTMaxOnTime", 39, "ALL"
            ),
            "CPOTMaxOffTime": GeckoWordStructAccessor(
                self.struct, "CPOTMaxOffTime", 41, "ALL"
            ),
            "FiltOTDuration24H": GeckoWordStructAccessor(
                self.struct, "FiltOTDuration24H", 43, "ALL"
            ),
            "FiltSuspendTime": GeckoByteStructAccessor(
                self.struct, "FiltSuspendTime", 45, "ALL"
            ),
            "O3Usage": GeckoEnumStructAccessor(
                self.struct,
                "O3Usage",
                16,
                None,
                ["Filter", "Always"],
                None,
                None,
                "ALL",
            ),
            "O3Pump": GeckoEnumStructAccessor(
                self.struct, "O3Pump", 17, None, ["CP", "P1"], None, None, "ALL"
            ),
            "O3Type": GeckoEnumStructAccessor(
                self.struct,
                "O3Type",
                18,
                None,
                ["Standard", "Toggle"],
                None,
                None,
                "ALL",
            ),
            "O3SuspendTime": GeckoByteStructAccessor(
                self.struct, "O3SuspendTime", 46, "ALL"
            ),
            "SetpointG": GeckoTempStructAccessor(self.struct, "SetpointG", 1, "ALL"),
            "HeaterPump": GeckoEnumStructAccessor(
                self.struct, "HeaterPump", 19, None, ["CP", "P1"], None, None, "ALL"
            ),
            "TempUnits": GeckoEnumStructAccessor(
                self.struct, "TempUnits", 21, None, ["F", "C"], None, None, "ALL"
            ),
            "MinSetpointG": GeckoTempStructAccessor(
                self.struct, "MinSetpointG", 49, "ALL"
            ),
            "MaxSetpointG": GeckoTempStructAccessor(
                self.struct, "MaxSetpointG", 51, "ALL"
            ),
            "EconType": GeckoEnumStructAccessor(
                self.struct,
                "EconType",
                53,
                None,
                ["Standard", "Night"],
                None,
                None,
                "ALL",
            ),
            "NbPhases": GeckoByteStructAccessor(self.struct, "NbPhases", 31, "ALL"),
            "InputCurrent": GeckoByteStructAccessor(
                self.struct, "InputCurrent", 32, "ALL"
            ),
            "InputMenu": GeckoEnumStructAccessor(
                self.struct,
                "InputMenu",
                57,
                None,
                ["Standard", "DualPack"],
                None,
                None,
                "ALL",
            ),
            "FiltFreq": GeckoByteStructAccessor(self.struct, "FiltFreq", 4, "ALL"),
            "FiltStart": GeckoTimeStructAccessor(self.struct, "FiltStart", 5, "ALL"),
            "FiltDur": GeckoTimeStructAccessor(self.struct, "FiltDur", 3, "ALL"),
            "EconStart": GeckoTimeStructAccessor(self.struct, "EconStart", 7, "ALL"),
            "EconDur": GeckoTimeStructAccessor(self.struct, "EconDur", 8, "ALL"),
            "EconProgAvailable": GeckoEnumStructAccessor(
                self.struct,
                "EconProgAvailable",
                54,
                None,
                ["NA", "Available"],
                None,
                None,
                "ALL",
            ),
            "SoakOnCustomKey": GeckoBoolStructAccessor(
                self.struct, "SoakOnCustomKey", 55, 0, "ALL"
            ),
            "OffOnCustomKey": GeckoBoolStructAccessor(
                self.struct, "OffOnCustomKey", 55, 1, "ALL"
            ),
            "EconControlableManually": GeckoBoolStructAccessor(
                self.struct, "EconControlableManually", 55, 2, "ALL"
            ),
            "MasterSlave": GeckoEnumStructAccessor(
                self.struct,
                "MasterSlave",
                56,
                None,
                ["Unconfigured", "Master", "Slave"],
                None,
                None,
                "ALL",
            ),
            "SlaveConfig": GeckoByteStructAccessor(
                self.struct, "SlaveConfig", 58, "ALL"
            ),
            "MultiKeyOption": GeckoEnumStructAccessor(
                self.struct,
                "MultiKeyOption",
                59,
                None,
                ["NoBlowerOnI2C", "BlowerOnI2C"],
                None,
                None,
                "ALL",
            ),
            "TimeFormat": GeckoEnumStructAccessor(
                self.struct,
                "TimeFormat",
                22,
                None,
                ["NA", "AmPm", "24h"],
                None,
                None,
                "ALL",
            ),
            "AmbiantOHTrigADC": GeckoWordStructAccessor(
                self.struct, "AmbiantOHTrigADC", 47, "ALL"
            ),
        }



#!/usr/bin/python3
"""
    GeckoConfigStruct - A class to manage the ConfigStruct for 'MAS-IBC-32K v1'
"""

from . import (
    GeckoByteStructAccessor,
    GeckoWordStructAccessor,
    GeckoTimeStructAccessor,
    GeckoBoolStructAccessor,
    GeckoEnumStructAccessor,
    GeckoTempStructAccessor,
)


class GeckoConfigStruct:
    def __init__(self, struct_):
        self.struct = struct_

    @property
    def version(self):
        return 1

    @property
    def output_keys(self):
        return []

    @property
    def accessors(self):
        return {
            "LL_Backrest": GeckoEnumStructAccessor(
                self.struct,
                "LL_Backrest",
                0,
                None,
                ["SINGLE", "DUAL", "DUAL_THERMOPLACE", "DUAL_LITESTREME"],
                None,
                None,
                None,
            ),
            "LL_Heater_1": GeckoEnumStructAccessor(
                self.struct,
                "LL_Heater_1",
                1,
                None,
                ["Intensity1", "Intensity2", "Intensity3", "Intensity4"],
                None,
                None,
                None,
            ),
            "LL_Heater_2": GeckoEnumStructAccessor(
                self.struct,
                "LL_Heater_2",
                2,
                None,
                ["Intensity1", "Intensity2", "Intensity3", "Intensity4"],
                None,
                None,
                None,
            ),
            "LL_Blower": GeckoEnumStructAccessor(
                self.struct,
                "LL_Blower",
                3,
                None,
                ["STANDARD", "HIGH", "LOW"],
                None,
                None,
                None,
            ),
            "LL_Chromo": GeckoEnumStructAccessor(
                self.struct,
                "LL_Chromo",
                4,
                None,
                ["NONE", "BASIC", "DELUXE"],
                None,
                None,
                None,
            ),
            "LL_Audio": GeckoEnumStructAccessor(
                self.struct, "LL_Audio", 5, None, ["NONE", "MP3"], None, None, None
            ),
            "LL_Menu": GeckoEnumStructAccessor(
                self.struct, "LL_Menu", 6, None, ["STANDARD", "HOTEL"], None, None, None
            ),
            "LL_ComfortJet": GeckoEnumStructAccessor(
                self.struct,
                "LL_ComfortJet",
                7,
                None,
                ["DEACTIVATE", "ACTIVATE"],
                None,
                None,
                None,
            ),
            "LL_Aromacloud": GeckoEnumStructAccessor(
                self.struct,
                "LL_Aromacloud",
                8,
                None,
                ["DEACTIVATE", "ACTIVATE"],
                None,
                None,
                None,
            ),
        }



""" Unit tests for the structure accessor class """

import struct
import unittest

from context import (
    GeckoByteStructAccessor,
    GeckoWordStructAccessor,
    GeckoBoolStructAccessor,
    GeckoEnumStructAccessor,
    GeckoStructure,
)


class TestStructAccessor(unittest.TestCase):
    """ Test the GeckoStructAccessor class """

    def setUp(self):
        # self.spa = MockSpa()
        self.struct = GeckoStructure(self._on_set_value)
        self.struct.set_status_block(
            # Bytes 0-16 are identity values
            b"\x00\x01\x02\x03\x04\x05\x06\x07\x08\x09\x0a\x0b\x0c\x0d\x0e\x0f\x10"
            # Bytes 17-18 are a temperature in farenheight 10ths from freezing
            b"\x02\xbe"
            # Bytes 19-20 are a sized bitpos enum
            b"\x11\x70"
            # Bytes 21-22 are an empty sized bitpos enum
            b"\x00\x00"
        )
        self.last_pos = None
        self.last_len = None
        self.last_data = None
        self.last_value = None

    def _on_set_value(self, pos, len_, newvalue):
        self.last_pos = pos
        self.last_len = len_
        self.last_data = struct.pack(">B" if len_ == 1 else ">H", newvalue)
        self.struct.replace_status_block_segment(pos, self.last_data)
        self.last_value = newvalue

    def test_read_byte(self):
        """ Can we read a byte from the structure """
        accessor = GeckoByteStructAccessor(self.struct, "PackBootRev", 5, None)
        self.assertEqual(5, accessor.value)

    @unittest.expectedFailure
    def test_write_byte_fails(self):
        """ Can we write a byte to the structure without the RW tag? """
        accessor = GeckoByteStructAccessor(self.struct, "PackBootRev", 5, None)
        accessor.value = 6

    def test_write_byte(self):
        """ Can we write a byte to the structure """
        accessor = GeckoByteStructAccessor(self.struct, "PackBootRev", 5, "ALL")
        accessor.value = 6
        self.assertEqual(5, self.last_pos)
        self.assertEqual(b"\x06", self.last_data)

    def test_write_byte_from_string(self):
        """ Can we write a byte (as a string) to the structure """
        accessor = GeckoByteStructAccessor(self.struct, "PackBootRev", 5, "ALL")
        accessor.value = "6"
        self.assertEqual(5, self.last_pos)
        self.assertEqual(b"\x06", self.last_data)

    def test_read_word(self):
        """ Can we read a word from the structure """
        accessor = GeckoWordStructAccessor(self.struct, "RealSetPointG", 17, None)
        self.assertEqual(702, accessor.value)

    def test_write_word(self):
        """ Can we write a word to the structure """
        accessor = GeckoWordStructAccessor(self.struct, "RealSetPointG", 17, "All")
        accessor.value = 726
        self.assertEqual(17, self.last_pos)
        self.assertEqual(b"\x02\xd6", self.last_data)

    def test_write_word_as_string(self):
        """ Can we write a word (as a string) to the structure """
        accessor = GeckoWordStructAccessor(self.struct, "RealSetPointG", 17, "All")
        accessor.value = "726"
        self.assertEqual(17, self.last_pos)
        self.assertEqual(b"\x02\xd6", self.last_data)

    def test_read_enum(self):
        """ Can we read an enum from the structure """
        accessor = GeckoEnumStructAccessor(
            self.struct,
            "PackType",
            6,
            None,
            [
                "Unknown",
                "inXE",
                "MasIBC",
                "MIA",
                "DJS4",
                "inClear",
                "inXM",
                "K600",
                "inTerface",
                "inTouch",
                "inYT",
                "K800",
                "inYJ",
            ],
            None,
            None,
            None,
        )
        self.assertEqual("inXM", accessor.value)

    def test_write_enum(self):
        """ Can we write an enum to the structure """
        accessor = GeckoEnumStructAccessor(
            self.struct,
            "PackType",
            6,
            None,
            (
                "Unknown|inXE|MasIBC|MIA|DJS4|inClear|inXM|K600"
                "|inTerface|inTouch|inYT|K800|inYJ"
            ),
            None,
            None,
            "All",
        )
        accessor.value = "inYJ"
        self.assertEqual(6, self.last_pos)
        self.assertEqual(b"\x0c", self.last_data)

    @unittest.expectedFailure
    def test_write_enum_not_member(self):
        """ Can we write an enum to the structure """
        accessor = GeckoEnumStructAccessor(
            self.struct,
            "PackType",
            6,
            None,
            (
                "Unknown|inXE|MasIBC|MIA|DJS4|inClear|inXM|K600"
                "|inTerface|inTouch|inYT|K800|inYJ"
            ),
            None,
            None,
            "All",
        )
        accessor.value = "Not A Member"

    def test_read_bool(self):
        """ Can we read a bool from the structure """
        accessor = GeckoBoolStructAccessor(self.struct, "RelayStuck", 2, 6, None)
        self.assertFalse(accessor.value)

    def test_write_bool(self):
        """ Can we write a bool to the structure """
        accessor = GeckoBoolStructAccessor(self.struct, "RelayStuck", 2, 6, "All")
        accessor.value = True
        self.assertEqual(2, self.last_pos)
        self.assertEqual(b"B", self.last_data)

    def test_write_bool_as_string(self):
        """ Can we write a bool (as a string) to the structure """
        accessor = GeckoBoolStructAccessor(self.struct, "RelayStuck", 2, 6, "All")
        accessor.value = "True"
        self.assertEqual(2, self.last_pos)
        self.assertEqual(b"B", self.last_data)

    def test_read_bitpos_enum(self):
        """ Can we read a bitpos enum from the structure """
        accessor = GeckoEnumStructAccessor(
            self.struct, "UdP3", 3, 4, "OFF|LO|HI", None, 4, None
        )
        self.assertEqual("OFF", accessor.value)

    def test_write_bitpos_enum(self):
        """ Can we write an enum to the structure """
        accessor = GeckoEnumStructAccessor(
            self.struct, "UdP3", 3, 4, "OFF|LO|HI", None, 4, "All"
        )
        accessor.value = "HI"
        self.assertEqual(3, self.last_pos)
        self.assertEqual(b"#", self.last_data)
        accessor.value = "LO"
        self.assertEqual(b"\x13", self.last_data)
        accessor.value = "OFF"
        self.assertEqual(b"\x03", self.last_data)

    def test_read_sized_bitpos_enum(self):
        """ Can we read a sized bitpos enum from the structure """
        accessor = GeckoEnumStructAccessor(
            self.struct, "UdP2", 19, 12, "OFF|LO|HI", 2, 4, None
        )
        self.assertEqual("LO", accessor.value)

    def test_write_sized_bitpos_enum(self):
        """ Can we write a sized bitpos enum to the structure """
        accessor = GeckoEnumStructAccessor(
            self.struct, "UdP2", 19, 12, "OFF|LO|HI", 2, 4, "All"
        )
        accessor.value = "HI"
        self.assertEqual(19, self.last_pos)
        self.assertEqual(
            chr(0b00100001) + chr(0b01110000), self.last_data.decode("latin-1")
        )
        accessor.value = "LO"
        self.assertEqual(
            chr(0b00010001) + chr(0b01110000), self.last_data.decode("latin-1")
        )
        accessor.value = "OFF"
        self.assertEqual(
            chr(0b00000001) + chr(0b01110000), self.last_data.decode("latin-1")
        )

    def test_multiple_write_bitpos_enum(self):
        """ Can we write multiple bitpos enums to the structure """

        accessor_p1 = GeckoEnumStructAccessor(
            self.struct, "UdP1", 21, 14, "OFF|LO|HI", 2, 4, "ALL"
        )
        accessor_p2 = GeckoEnumStructAccessor(
            self.struct, "UdP2", 21, 12, "OFF|LO|HI", 2, 4, "All"
        )

        accessor_p2.value = "HI"
        self.assertEqual(21, self.last_pos)
        self.assertEqual(b"\x20\x00", self.last_data)
        self.assertEqual(2, self.last_len)
        self.assertEqual(2, len(self.last_data))

        accessor_p1.value = "HI"
        self.assertEqual(21, self.last_pos)
        self.assertEqual(b"\xA0\x00", self.last_data)
        self.assertEqual(2, self.last_len)
        self.assertEqual(2, len(self.last_data))


if __name__ == "__main__":
    unittest.main()



""" Unit tests for the structure class """

from context import (
    GeckoByteStructAccessor,
    GeckoWordStructAccessor,
    GeckoBoolStructAccessor,
    GeckoEnumStructAccessor,
    GeckoStructure,
)

import unittest
import unittest.mock


class TestStruct(unittest.TestCase):
    """ Test the GeckoStruct class """

    def setUp(self):
        self.struct = GeckoStructure(None)

        class log_class:
            def __init__(self, struct_):
                self.struct = struct_

            @property
            def temperature_keys(self):
                return []

            @property
            def output_keys(self):
                return []

            @property
            def accessors(self):
                return {
                    # <QuietState Type="Enum" Pos="362" Items="NOT_SET|QUIET|DRAIN|SOAK|OFF" RW="ALL" />
                    "QuietState": GeckoEnumStructAccessor(
                        self.struct,
                        "QuietState",
                        362,
                        None,
                        "NOT_SET|QUIET|DRAIN|SOAK|OFF",
                        None,
                        None,
                        "All",
                    ),
                    # <UdP1 Type="Enum" Pos="275" BitPos="14" Size="2" MaxItems="4" Items="OFF|LO|HI" RW="ALL" />
                    "UdP1": GeckoEnumStructAccessor(
                        self.struct, "UdP1", 275, 14, "OFF|LO|HI", 2, 4, "All"
                    ),
                    # <UdP2 Type="Enum" Pos="275" BitPos="12" Size="2" MaxItems="4" Items="OFF|LO|HI" RW="ALL" />
                    "UdP2": GeckoEnumStructAccessor(
                        self.struct, "UdP2", 275, 12, "OFF|LO|HI", 2, 4, "All"
                    ),
                    # <UdP3 Type="Enum" Pos="275" BitPos="11" Size="2" MaxItems="2" Items="OFF|HI" RW="ALL" />
                    "UdP3": GeckoEnumStructAccessor(
                        self.struct, "UdP3", 275, 11, "OFF|HI", 2, 2, "All"
                    ),
                    # <UdP4 Type="Enum" Pos="275" BitPos="10" Size="2" MaxItems="2" Items="OFF|HI" RW="ALL" />
                    "UdP4": GeckoEnumStructAccessor(
                        self.struct, "UdP4", 275, 10, "OFF|HI", 2, 2, "All"
                    ),
                    # <UdP5 Type="Enum" Pos="275" BitPos="9" Size="2" MaxItems="2" Items="OFF|HI" RW="ALL" />
                    "UdP5": GeckoEnumStructAccessor(
                        self.struct, "UdP5", 275, 9, "OFF|HI", 2, 2, "All"
                    ),
                    # <UdBL Type="Enum" Pos="275" BitPos="8" Size="2" MaxItems="2" Items="OFF|ON" RW="ALL" />
                    "UdBL": GeckoEnumStructAccessor(
                        self.struct, "UdBL", 275, 8, "OFF|ON", 2, 2, "All"
                    ),
                    # <UdL120 Type="Enum" Pos="275" BitPos="1" Size="2" MaxItems="2" Items="OFF|ON" RW="ALL" />
                    "UdL120": GeckoEnumStructAccessor(
                        self.struct, "UdL120", 275, 1, "OFF|ON", 2, 2, "All"
                    ),
                    # <UdLi Type="Enum" Pos="306" BitPos="0" MaxItems="4" Items="OFF|LO|MED|HI" RW="ALL" />
                    "UdLi": GeckoEnumStructAccessor(
                        self.struct, "UdLi", 306, 0, "OFF|LO|MED|HI", None, 4, "All"
                    ),
                }

        class cfg_class:
            def __init__(self, struct_):
                self.struct = struct_

            @property
            def temperature_keys(self):
                return []

            @property
            def all_device_keys(self):
                return []

            @property
            def user_demand_keys(self):
                return []

            @property
            def error_keys(self):
                return []

            @property
            def accessors(self):
                return {
                    # <UdValve Type="Enum" Pos="275" BitPos="7" Size="2" MaxItems="2" Items="OFF|ON" RW="ALL" />
                    # <UdTvLift Type="Enum" Pos="275" BitPos="5" Size="2" MaxItems="2" Items="OFF|ON" RW="ALL" />
                    # <UdSpkrLift Type="Enum" Pos="275" BitPos="4" Size="2" MaxItems="2" Items="OFF|ON" RW="ALL" />
                    # <UdVSP1 Type="Byte" Pos="303" Min="0" Max="100" RW="ALL" />
                    "UdVSP1": GeckoByteStructAccessor(
                        self.struct, "UdVSP1", 303, "All"
                    ),
                    # <UdVSP3 Type="Byte" Pos="304" Min="0" Max="100" RW="ALL" />
                    # <UdPumpTime Type="Byte" Pos="358" RW="ALL" />
                    # <UdQuietTime Type="Byte" Pos="359" Max="90" RW="ALL" />
                    # <UdLightTime Type="Byte" Pos="360" RW="ALL" />
                    # <UdL120Time Type="Byte" Pos="361" RW="ALL" />
                }

        self.struct.build_accessors(log_class(self.struct), cfg_class(self.struct))

        self.struct._on_set_value = self.set_value_cb
        self.last_pos = None
        self.last_len = None
        self.last_value = None

    def set_value_cb(self, pos, len_, newvalue):
        self.last_pos = pos
        self.last_len = len_
        self.last_value = newvalue

    def test_constructor(self):
        self.assertEqual(self.struct.status_block, b"\0" * 1024)
        self.assertEqual(len(self.struct.accessors), 10)
        self.assertEqual(self.struct.accessors["UdL120"].value, "OFF")

    def test_set_value_callback(self):
        self.struct.accessors["UdP1"].value = "HI"
        self.assertEqual(self.last_pos, 275)
        self.assertEqual(self.last_len, 2)
        self.assertEqual(self.last_value, 0b1000000000000000)

    def test_observer_callback(self):
        P2_info = {"oldvalue": None, "newvalue": None}

        def P2_Watch(sender, oldvalue, newvalue):
            P2_info["oldvalue"] = oldvalue
            P2_info["newvalue"] = newvalue

        self.struct.accessors["UdP2"].watch(P2_Watch)
        self.struct.replace_status_block_segment(275, b"\x20\x00")
        self.assertEqual(P2_info["newvalue"], "HI")


if __name__ == "__main__":
    unittest.main()



""" Gecko REQRM/RMREQ handlers """
from __future__ import annotations

import logging
import struct

from ...config import GeckoConfig
from enum import IntEnum
from .packet import GeckoPacketProtocolHandler
from typing import List, Tuple

REQRM_VERB = b"REQRM"
RMREQ_VERB = b"RMREQ"

RESPONSE_FORMAT = ">BBB"

_LOGGER = logging.getLogger(__name__)


class GeckoReminderType(IntEnum):
    INVALID = 0
    RINSE_FILTER = 1
    CLEAN_FILTER = 2
    CHANGE_WATER = 3
    CHECK_SPA = 4
    CHANGE_OZONATOR = 5
    CHANGE_VISION_CARTRIDGE = 6

    @staticmethod
    def to_string(type: GeckoReminderType) -> str:
        if type == GeckoReminderType.INVALID:
            return "Invalid"
        elif type == GeckoReminderType.RINSE_FILTER:
            return "RinseFilter"
        elif type == GeckoReminderType.CLEAN_FILTER:
            return "CleanFilter"
        elif type == GeckoReminderType.CHANGE_WATER:
            return "ChangeWater"
        elif type == GeckoReminderType.CHECK_SPA:
            return "CheckSpa"
        elif type == GeckoReminderType.CHANGE_OZONATOR:
            return "ChangeOzonator"
        elif type == GeckoReminderType.CHANGE_VISION_CARTRIDGE:
            return "ChangeVisionCartridge"
        else:
            # Technically unreachable code here
            return "Unhandled"


class GeckoRemindersProtocolHandler(GeckoPacketProtocolHandler):
    @staticmethod
    def request(seq, **kwargs):
        return GeckoRemindersProtocolHandler(
            content=b"".join([REQRM_VERB, struct.pack(">B", seq)]),
            timeout=GeckoConfig.PROTOCOL_TIMEOUT_IN_SECONDS,
            retry_count=GeckoConfig.PROTOCOL_RETRY_COUNT,
            on_retry_failed=GeckoPacketProtocolHandler._default_retry_failed_handler,
            **kwargs,
        )

    @staticmethod
    def response(reminders: List[Tuple[GeckoReminderType, int]], **kwargs):
        return GeckoRemindersProtocolHandler(
            content=b"".join(
                [RMREQ_VERB]
                + [
                    struct.pack("<BhB", reminder[0], reminder[1], 1)
                    for reminder in reminders
                ]
            ),
            **kwargs,
        )

    def __init__(self, **kwargs):
        super().__init__(**kwargs)
        self.reminders: List[Tuple[GeckoReminderType, ...]] = []

    def can_handle(self, received_bytes: bytes, sender: tuple) -> bool:
        return received_bytes.startswith(REQRM_VERB) or received_bytes.startswith(
            RMREQ_VERB
        )

    def handle(self, received_bytes: bytes, sender: tuple):
        remainder = received_bytes[5:]
        if received_bytes.startswith(REQRM_VERB):
            self._sequence = struct.unpack(">B", remainder[0:1])[0]
            return  # Stay in the handler list

        # Otherwise must be RMREQ
        rest = remainder
        while len(rest) > 0:
            (t, days, _push, rest) = struct.unpack(
                "<BhB{}s".format(len(rest) - 4), rest
            )
            try:
                self.reminders.append(tuple((GeckoReminderType(t), days)))
            except ValueError:
                _LOGGER.warning("Cannot use %d as reminder type, ignored", t)

        self._should_remove_handler = True



#!/usr/bin/python3
"""
    GeckoLogStruct - A class to manage the LogStruct for 'InXM v7'
"""

from . import (
    GeckoByteStructAccessor,
    GeckoWordStructAccessor,
    GeckoTimeStructAccessor,
    GeckoBoolStructAccessor,
    GeckoEnumStructAccessor,
    GeckoTempStructAccessor,
)


class GeckoLogStruct:
    def __init__(self, struct_):
        self.struct = struct_

    @property
    def version(self):
        return 7

    @property
    def begin(self):
        return 256

    @property
    def end(self):
        return 479

    @property
    def all_device_keys(self):
        return [
            "P1",
            "P2",
            "P3",
            "P4",
            "P5",
            "BL",
            "CP",
            "O3",
            "L120",
            "FAN",
            "Fb",
            "AlwayOn",
            "TvLift",
            "SpkrLift",
            "Sani",
            "Onzen",
            "Valve",
            "RhDuty",
            "Vsp1Front",
            "Vsp1Min",
            "VSP3Front",
            "LockMode",
            "LI",
        ]

    @property
    def user_demand_keys(self):
        return [
            "UdP1",
            "UdP2",
            "UdP3",
            "UdP4",
            "UdP5",
            "UdBL",
            "UdL120",
            "UdLi",
            "UdValve",
            "UdTvLift",
            "UdSpkrLift",
            "UdFb",
            "UdVSP1",
            "UdVSP3",
            "UdPumpTime",
            "UdQuietTime",
            "UdLightTime",
            "UdL120Time",
        ]

    @property
    def error_keys(self):
        return [
            "AmbiantOHLevel2",
            "RegOverHeat",
            "RhDutyErr",
            "ThermistanceErr",
            "LearnPhaseErr",
            "rHId",
            "Fuse3Err",
            "SupplyErr",
            "Fuse2Err",
            "TempNotValid",
            "RhCommErr",
            "FreqDetecErr",
            "KinPumpOff",
            "ScanErr",
            "ContactorCoilFailErr",
            "RelayStuck",
            "RhRegProbeErr",
            "NoHeaterCurrentErr",
            "AmbiantOHLevel1",
            "Fuse1Err",
        ]

    @property
    def accessors(self):
        return {
            "RhWaterTemp": GeckoTempStructAccessor(
                self.struct, "RhWaterTemp", 261, None
            ),
            "RhTriacTemp": GeckoTempStructAccessor(
                self.struct, "RhTriacTemp", 263, None
            ),
            "Hours": GeckoByteStructAccessor(self.struct, "Hours", 256, None),
            "Menu": GeckoEnumStructAccessor(
                self.struct,
                "Menu",
                353,
                None,
                [
                    "NORMAL",
                    "S1",
                    "S2",
                    "H1",
                    "H2",
                    "H3",
                    "H4",
                    "Fr",
                    "LearnFail",
                    "Learning",
                    "BreakerSelect",
                    "PhaseSelect",
                    "ConfigSelect",
                    "StickBank",
                    "K600Upload",
                    "Powerup",
                ],
                None,
                None,
                None,
            ),
            "HeaterPhaseSign": GeckoByteStructAccessor(
                self.struct, "HeaterPhaseSign", 300, None
            ),
            "Out1ARead": GeckoWordStructAccessor(self.struct, "Out1ARead", 280, None),
            "Out1BRead": GeckoWordStructAccessor(self.struct, "Out1BRead", 282, None),
            "Out2ARead": GeckoWordStructAccessor(self.struct, "Out2ARead", 284, None),
            "Out2BRead": GeckoWordStructAccessor(self.struct, "Out2BRead", 286, None),
            "Out3ARead": GeckoWordStructAccessor(self.struct, "Out3ARead", 288, None),
            "Out4ARead": GeckoWordStructAccessor(self.struct, "Out4ARead", 290, None),
            "Out5ARead": GeckoWordStructAccessor(self.struct, "Out5ARead", 292, None),
            "Out6ARead": GeckoWordStructAccessor(self.struct, "Out6ARead", 294, None),
            "Out7ARead": GeckoWordStructAccessor(self.struct, "Out7ARead", 296, None),
            "OutHtrRead": GeckoWordStructAccessor(self.struct, "OutHtrRead", 298, None),
            "QuietState": GeckoEnumStructAccessor(
                self.struct,
                "QuietState",
                362,
                None,
                ["NOT_SET", "QUIET", "DRAIN", "SOAK", "OFF"],
                None,
                None,
                "ALL",
            ),
            "UdP1": GeckoEnumStructAccessor(
                self.struct, "UdP1", 275, 14, ["OFF", "LO", "HI"], 2, 4, "ALL"
            ),
            "UdP2": GeckoEnumStructAccessor(
                self.struct, "UdP2", 275, 12, ["OFF", "LO", "HI"], 2, 4, "ALL"
            ),
            "UdP3": GeckoEnumStructAccessor(
                self.struct, "UdP3", 275, 11, ["OFF", "HI"], 2, 2, "ALL"
            ),
            "UdP4": GeckoEnumStructAccessor(
                self.struct, "UdP4", 275, 10, ["OFF", "HI"], 2, 2, "ALL"
            ),
            "UdP5": GeckoEnumStructAccessor(
                self.struct, "UdP5", 275, 9, ["OFF", "HI"], 2, 2, "ALL"
            ),
            "UdBL": GeckoEnumStructAccessor(
                self.struct, "UdBL", 275, 8, ["OFF", "ON"], 2, 2, "ALL"
            ),
            "UdL120": GeckoEnumStructAccessor(
                self.struct, "UdL120", 275, 1, ["OFF", "ON"], 2, 2, "ALL"
            ),
            "UdLi": GeckoEnumStructAccessor(
                self.struct, "UdLi", 306, 0, ["OFF", "LO", "MED", "HI"], None, 4, "ALL"
            ),
            "UdValve": GeckoEnumStructAccessor(
                self.struct, "UdValve", 275, 7, ["OFF", "ON"], 2, 2, "ALL"
            ),
            "UdTvLift": GeckoEnumStructAccessor(
                self.struct, "UdTvLift", 275, 5, ["OFF", "ON"], 2, 2, "ALL"
            ),
            "UdSpkrLift": GeckoEnumStructAccessor(
                self.struct, "UdSpkrLift", 275, 4, ["OFF", "ON"], 2, 2, "ALL"
            ),
            "UdFb": GeckoEnumStructAccessor(
                self.struct, "UdFb", 275, 2, ["OFF", "FIX", "NA", "SCAN"], 2, 4, "ALL"
            ),
            "UdVSP1": GeckoByteStructAccessor(self.struct, "UdVSP1", 303, "ALL"),
            "UdVSP3": GeckoByteStructAccessor(self.struct, "UdVSP3", 304, "ALL"),
            "UdPumpTime": GeckoByteStructAccessor(
                self.struct, "UdPumpTime", 358, "ALL"
            ),
            "UdQuietTime": GeckoByteStructAccessor(
                self.struct, "UdQuietTime", 359, "ALL"
            ),
            "UdLightTime": GeckoByteStructAccessor(
                self.struct, "UdLightTime", 360, "ALL"
            ),
            "UdL120Time": GeckoByteStructAccessor(
                self.struct, "UdL120Time", 361, "ALL"
            ),
            "P1": GeckoEnumStructAccessor(
                self.struct, "P1", 356, 0, ["OFF", "HIGH", "LOW"], None, 4, None
            ),
            "P2": GeckoEnumStructAccessor(
                self.struct, "P2", 356, 2, ["OFF", "HIGH", "LOW"], None, 4, None
            ),
            "P3": GeckoEnumStructAccessor(
                self.struct, "P3", 356, 4, ["OFF", "HIGH", "LOW"], None, 4, None
            ),
            "P4": GeckoEnumStructAccessor(
                self.struct, "P4", 356, 6, ["OFF", "HIGH", "LOW"], None, 4, None
            ),
            "P5": GeckoEnumStructAccessor(
                self.struct, "P5", 355, 0, ["OFF", "HIGH"], None, 2, None
            ),
            "BL": GeckoEnumStructAccessor(
                self.struct, "BL", 355, 1, ["OFF", "ON"], None, 2, None
            ),
            "CP": GeckoEnumStructAccessor(
                self.struct, "CP", 355, 2, ["OFF", "ON"], None, 2, None
            ),
            "O3": GeckoEnumStructAccessor(
                self.struct, "O3", 355, 3, ["OFF", "ON"], None, 2, None
            ),
            "L120": GeckoEnumStructAccessor(
                self.struct, "L120", 355, 4, ["OFF", "ON"], None, 2, None
            ),
            "FAN": GeckoEnumStructAccessor(
                self.struct, "FAN", 355, 7, ["OFF", "ON"], None, 2, None
            ),
            "Fb": GeckoEnumStructAccessor(
                self.struct, "Fb", 354, 0, ["OFF", "OFF", "FIX", "SCAN"], None, 4, None
            ),
            "AlwayOn": GeckoEnumStructAccessor(
                self.struct, "AlwayOn", 354, 2, ["OFF", "ON"], None, 2, None
            ),
            "TvLift": GeckoEnumStructAccessor(
                self.struct, "TvLift", 354, 3, ["DOWN", "UP"], None, 2, None
            ),
            "SpkrLift": GeckoEnumStructAccessor(
                self.struct, "SpkrLift", 354, 4, ["DOWN", "UP"], None, 2, None
            ),
            "Sani": GeckoEnumStructAccessor(
                self.struct, "Sani", 354, 5, ["OFF", "ON"], None, 2, None
            ),
            "Onzen": GeckoEnumStructAccessor(
                self.struct, "Onzen", 354, 6, ["OFF", "ON"], None, 2, None
            ),
            "Valve": GeckoEnumStructAccessor(
                self.struct, "Valve", 354, 7, ["OFF", "ON"], None, 2, None
            ),
            "RhDuty": GeckoByteStructAccessor(self.struct, "RhDuty", 274, None),
            "Vsp1Front": GeckoByteStructAccessor(self.struct, "Vsp1Front", 301, None),
            "Vsp1Min": GeckoByteStructAccessor(self.struct, "Vsp1Min", 305, None),
            "VSP3Front": GeckoByteStructAccessor(self.struct, "VSP3Front", 302, None),
            "LockMode": GeckoEnumStructAccessor(
                self.struct,
                "LockMode",
                363,
                None,
                ["UNLOCK", "PARTIAL", "FULL"],
                None,
                None,
                "ALL",
            ),
            "FilterAccess": GeckoEnumStructAccessor(
                self.struct,
                "FilterAccess",
                348,
                0,
                ["INTERNAL", "REMOTE"],
                None,
                2,
                "ALL",
            ),
            "RemoteFiltAction": GeckoEnumStructAccessor(
                self.struct,
                "RemoteFiltAction",
                307,
                None,
                ["IDLE", "STOP", "START", "NEW", "ACTIVE"],
                None,
                None,
                "ALL",
            ),
            "RemoteFiltDur": GeckoTimeStructAccessor(
                self.struct, "RemoteFiltDur", 308, "ALL"
            ),
            "RemoteFiltDurPerDay": GeckoByteStructAccessor(
                self.struct, "RemoteFiltDurPerDay", 316, "ALL"
            ),
            "OnzenAccess": GeckoEnumStructAccessor(
                self.struct,
                "OnzenAccess",
                348,
                1,
                ["INTERNAL", "REMOTE"],
                None,
                2,
                "ALL",
            ),
            "RemoteOnzenAction": GeckoEnumStructAccessor(
                self.struct,
                "RemoteOnzenAction",
                310,
                None,
                ["IDLE", "STOP", "START", "NEW", "ACTIVE"],
                None,
                None,
                "ALL",
            ),
            "RemoteOnzenDur": GeckoTimeStructAccessor(
                self.struct, "RemoteOnzenDur", 311, "ALL"
            ),
            "EconomyAccess": GeckoEnumStructAccessor(
                self.struct,
                "EconomyAccess",
                348,
                2,
                ["INTERNAL", "REMOTE"],
                None,
                2,
                "ALL",
            ),
            "RemoteEconAction": GeckoEnumStructAccessor(
                self.struct,
                "RemoteEconAction",
                313,
                None,
                ["IDLE", "STOP", "START", "NEW", "ACTIVE"],
                None,
                None,
                "ALL",
            ),
            "RemoteEconDur": GeckoTimeStructAccessor(
                self.struct, "RemoteEconDur", 314, "ALL"
            ),
            "RemoteConfigIndex": GeckoByteStructAccessor(
                self.struct, "RemoteConfigIndex", 350, "ALL"
            ),
            "RemoteNbOfPhases": GeckoByteStructAccessor(
                self.struct, "RemoteNbOfPhases", 351, "ALL"
            ),
            "RemoteBreakerIndex": GeckoByteStructAccessor(
                self.struct, "RemoteBreakerIndex", 352, "ALL"
            ),
            "Clean": GeckoBoolStructAccessor(self.struct, "Clean", 269, 5, None),
            "Purge": GeckoBoolStructAccessor(self.struct, "Purge", 269, 7, None),
            "FiltSuspendByUD": GeckoBoolStructAccessor(
                self.struct, "FiltSuspendByUD", 279, 3, None
            ),
            "FiltSuspendedByOT": GeckoBoolStructAccessor(
                self.struct, "FiltSuspendedByOT", 279, 0, None
            ),
            "CPOT": GeckoBoolStructAccessor(self.struct, "CPOT", 279, 1, None),
            "SwmPurgeSusp": GeckoBoolStructAccessor(
                self.struct, "SwmPurgeSusp", 269, 2, None
            ),
            "SwmPurge": GeckoBoolStructAccessor(self.struct, "SwmPurge", 269, 3, None),
            "SwmActive": GeckoBoolStructAccessor(
                self.struct, "SwmActive", 267, None, None
            ),
            "SwmRisk": GeckoEnumStructAccessor(
                self.struct,
                "SwmRisk",
                268,
                None,
                ["NO", "LO", "MED", "HI", "EXTREME"],
                None,
                None,
                None,
            ),
            "RealSetPointG": GeckoTempStructAccessor(
                self.struct, "RealSetPointG", 272, None
            ),
            "DisplayedTempG": GeckoTempStructAccessor(
                self.struct, "DisplayedTempG", 265, None
            ),
            "Heating": GeckoBoolStructAccessor(self.struct, "Heating", 270, 7, None),
            "TempNotValid": GeckoBoolStructAccessor(
                self.struct, "TempNotValid", 349, 1, None
            ),
            "ExtProbeDetected": GeckoBoolStructAccessor(
                self.struct, "ExtProbeDetected", 364, 0, None
            ),
            "CheckFlo": GeckoBoolStructAccessor(self.struct, "CheckFlo", 269, 0, None),
            "ProgEconActive": GeckoBoolStructAccessor(
                self.struct, "ProgEconActive", 271, 2, None
            ),
            "EconActive": GeckoBoolStructAccessor(
                self.struct, "EconActive", 271, 0, "ALL"
            ),
            "PumpRun": GeckoBoolStructAccessor(self.struct, "PumpRun", 269, 1, None),
            "CoolingDown": GeckoBoolStructAccessor(
                self.struct, "CoolingDown", 270, 6, None
            ),
            "KinPurgeActive": GeckoBoolStructAccessor(
                self.struct, "KinPurgeActive", 270, 5, None
            ),
            "ThermistanceErr": GeckoBoolStructAccessor(
                self.struct, "ThermistanceErr", 277, 5, None
            ),
            "AmbiantOHLevel2": GeckoBoolStructAccessor(
                self.struct, "AmbiantOHLevel2", 278, 3, None
            ),
            "AmbiantOHLevel1": GeckoBoolStructAccessor(
                self.struct, "AmbiantOHLevel1", 278, 2, None
            ),
            "KinPumpOff": GeckoBoolStructAccessor(
                self.struct, "KinPumpOff", 279, 4, None
            ),
            "RegOverHeat": GeckoBoolStructAccessor(
                self.struct, "RegOverHeat", 278, 4, None
            ),
            "RelayStuck": GeckoBoolStructAccessor(
                self.struct, "RelayStuck", 278, 1, None
            ),
            "FreqDetecErr": GeckoBoolStructAccessor(
                self.struct, "FreqDetecErr", 277, 6, None
            ),
            "ContactorCoilFailErr": GeckoBoolStructAccessor(
                self.struct, "ContactorCoilFailErr", 277, 4, None
            ),
            "NoHeaterCurrentErr": GeckoBoolStructAccessor(
                self.struct, "NoHeaterCurrentErr", 277, 3, None
            ),
            "LearnPhaseErr": GeckoBoolStructAccessor(
                self.struct, "LearnPhaseErr", 277, 2, None
            ),
            "Fuse3Err": GeckoBoolStructAccessor(self.struct, "Fuse3Err", 277, 1, None),
            "Fuse2Err": GeckoBoolStructAccessor(self.struct, "Fuse2Err", 277, 0, None),
            "Fuse1Err": GeckoBoolStructAccessor(self.struct, "Fuse1Err", 278, 7, None),
            "SupplyErr": GeckoBoolStructAccessor(
                self.struct, "SupplyErr", 278, 6, None
            ),
            "ScanErr": GeckoBoolStructAccessor(self.struct, "ScanErr", 278, 5, None),
            "rHId": GeckoBoolStructAccessor(self.struct, "rHId", 278, 0, None),
            "RhFloDetected": GeckoBoolStructAccessor(
                self.struct, "RhFloDetected", 260, 3, None
            ),
            "RhHwHL": GeckoBoolStructAccessor(self.struct, "RhHwHL", 258, 2, None),
            "RhRegProbeErr": GeckoBoolStructAccessor(
                self.struct, "RhRegProbeErr", 258, 1, None
            ),
            "RhRegSlope": GeckoBoolStructAccessor(
                self.struct, "RhRegSlope", 257, 2, None
            ),
            "RhHrKinNoFlo": GeckoBoolStructAccessor(
                self.struct, "RhHrKinNoFlo", 259, 2, None
            ),
            "RhNoFloXTries": GeckoBoolStructAccessor(
                self.struct, "RhNoFloXTries", 258, 4, None
            ),
            "inTCipDelay": GeckoWordStructAccessor(
                self.struct, "inTCipDelay", 365, None
            ),
            "RhCommErr": GeckoBoolStructAccessor(
                self.struct, "RhCommErr", 257, 7, None
            ),
            "RhNotEnoughHeat": GeckoBoolStructAccessor(
                self.struct, "RhNotEnoughHeat", 257, 3, None
            ),
            "RhLowFlo": GeckoBoolStructAccessor(self.struct, "RhLowFlo", 257, 0, None),
            "RhDutyErr": GeckoBoolStructAccessor(
                self.struct, "RhDutyErr", 258, 7, None
            ),
            "RhNoFloRetry": GeckoBoolStructAccessor(
                self.struct, "RhNoFloRetry", 258, 6, None
            ),
            "RhNoFloTemp": GeckoBoolStructAccessor(
                self.struct, "RhNoFloTemp", 258, 5, None
            ),
            "RhHrHL": GeckoBoolStructAccessor(self.struct, "RhHrHL", 258, 3, None),
            "RhRegOverHeat": GeckoBoolStructAccessor(
                self.struct, "RhRegOverHeat", 258, 0, None
            ),
            "RhHrTriacPr": GeckoBoolStructAccessor(
                self.struct, "RhHrTriacPr", 259, 7, None
            ),
            "RhHrTriacOH": GeckoBoolStructAccessor(
                self.struct, "RhHrTriacOH", 259, 6, None
            ),
            "RhSWTriacOH": GeckoBoolStructAccessor(
                self.struct, "RhSWTriacOH", 259, 5, None
            ),
            "RhHwTriacOH": GeckoBoolStructAccessor(
                self.struct, "RhHwTriacOH", 259, 4, None
            ),
            "RhHrKin": GeckoBoolStructAccessor(self.struct, "RhHrKin", 259, 3, None),
            "RhSwKinTemp": GeckoBoolStructAccessor(
                self.struct, "RhSwKinTemp", 259, 1, None
            ),
            "RhHWKin": GeckoBoolStructAccessor(self.struct, "RhHWKin", 259, 0, None),
            "RhHeaterDisabled": GeckoBoolStructAccessor(
                self.struct, "RhHeaterDisabled", 260, 4, None
            ),
            "RhFloCheck": GeckoBoolStructAccessor(
                self.struct, "RhFloCheck", 260, 2, None
            ),
            "RhHeaterSuspended": GeckoBoolStructAccessor(
                self.struct, "RhHeaterSuspended", 260, 1, None
            ),
            "RhFloCheckReady": GeckoBoolStructAccessor(
                self.struct, "RhFloCheckReady", 260, 0, None
            ),
            "PackBootID": GeckoWordStructAccessor(self.struct, "PackBootID", 317, None),
            "PackBootRev": GeckoByteStructAccessor(
                self.struct, "PackBootRev", 319, None
            ),
            "PackBootRel": GeckoByteStructAccessor(
                self.struct, "PackBootRel", 320, None
            ),
            "PackType": GeckoEnumStructAccessor(
                self.struct,
                "PackType",
                321,
                None,
                [
                    "Unknown",
                    "inXE",
                    "MasIBC",
                    "MIA",
                    "DJS4",
                    "inClear",
                    "inXM",
                    "K600",
                    "inTerface",
                    "inTouch",
                    "inYT",
                ],
                None,
                None,
                None,
            ),
            "PackMemRange": GeckoEnumStructAccessor(
                self.struct,
                "PackMemRange",
                322,
                0,
                ["16K", "32K", "48K", "64K"],
                None,
                4,
                None,
            ),
            "PackRegion": GeckoEnumStructAccessor(
                self.struct, "PackRegion", 322, 2, ["UL", "CE"], None, 2, None
            ),
            "PackCoreID": GeckoWordStructAccessor(self.struct, "PackCoreID", 325, None),
            "PackCoreRev": GeckoByteStructAccessor(
                self.struct, "PackCoreRev", 327, None
            ),
            "PackCoreRel": GeckoByteStructAccessor(
                self.struct, "PackCoreRel", 328, None
            ),
            "PackConfigLib": GeckoByteStructAccessor(
                self.struct, "PackConfigLib", 324, None
            ),
            "PackStatusLib": GeckoByteStructAccessor(
                self.struct, "PackStatusLib", 323, None
            ),
            "PackConfID": GeckoWordStructAccessor(self.struct, "PackConfID", 329, None),
            "PackConfRev": GeckoByteStructAccessor(
                self.struct, "PackConfRev", 331, None
            ),
            "PackConfRel": GeckoByteStructAccessor(
                self.struct, "PackConfRel", 332, None
            ),
            "PackNumberOfConf": GeckoWordStructAccessor(
                self.struct, "PackNumberOfConf", 333, None
            ),
            "PackLogTrig": GeckoEnumStructAccessor(
                self.struct,
                "PackLogTrig",
                357,
                None,
                ["Restricted", "Full"],
                None,
                2,
                "ALL",
            ),
            "inThermID": GeckoWordStructAccessor(self.struct, "inThermID", 335, None),
            "inThermRev": GeckoByteStructAccessor(self.struct, "inThermRev", 337, None),
            "inThermRel": GeckoByteStructAccessor(self.struct, "inThermRel", 338, None),
            "inThermCur1KW": GeckoWordStructAccessor(
                self.struct, "inThermCur1KW", 339, None
            ),
            "inThermCurFull": GeckoWordStructAccessor(
                self.struct, "inThermCurFull", 341, None
            ),
        }



#!/usr/bin/python3
"""
    GeckoLogStruct - A class to manage the LogStruct for 'MAS-IBC-32K v1'
"""

from . import (
    GeckoByteStructAccessor,
    GeckoWordStructAccessor,
    GeckoTimeStructAccessor,
    GeckoBoolStructAccessor,
    GeckoEnumStructAccessor,
    GeckoTempStructAccessor,
)


class GeckoLogStruct:
    def __init__(self, struct_):
        self.struct = struct_

    @property
    def version(self):
        return 1

    @property
    def begin(self):
        return 256

    @property
    def end(self):
        return 285

    @property
    def all_device_keys(self):
        return [
            "KeypadID",
            "KeypadRev",
            "BlowerState",
            "PurgeStart",
            "PurgeDelayTimer",
            "PurgeStandby",
            "PowerState",
            "KeypadLock",
            "LI",
        ]

    @property
    def user_demand_keys(self):
        return []

    @property
    def error_keys(self):
        return []

    @property
    def accessors(self):
        return {
            "UserBlowerIntensity": GeckoByteStructAccessor(
                self.struct, "UserBlowerIntensity", 256, "ALL"
            ),
            "UserChroma": GeckoEnumStructAccessor(
                self.struct,
                "UserChroma",
                257,
                None,
                [
                    "OFF",
                    "WHITE",
                    "RED",
                    "GREEN",
                    "BLUE",
                    "PURPLE",
                    "YELLOW",
                    "SCAN",
                    "ORANGE",
                    "PAUSE",
                ],
                None,
                None,
                "ALL",
            ),
            "UserBathTime": GeckoByteStructAccessor(
                self.struct, "UserBathTime", 258, "ALL"
            ),
            "UserHeater1": GeckoEnumStructAccessor(
                self.struct,
                "UserHeater1",
                259,
                None,
                ["OFF", "LOW", "MED", "HIGH"],
                None,
                None,
                "ALL",
            ),
            "UserHeater2": GeckoEnumStructAccessor(
                self.struct,
                "UserHeater2",
                260,
                None,
                ["OFF", "LOW", "MED", "HIGH"],
                None,
                None,
                "ALL",
            ),
            "UserGeysair": GeckoEnumStructAccessor(
                self.struct, "UserGeysair", 261, None, ["OFF", "ON"], None, None, None
            ),
            "UserDryingCycle": GeckoEnumStructAccessor(
                self.struct,
                "UserDryingCycle",
                262,
                0,
                ["AfterBath", "Daily"],
                None,
                2,
                "ALL",
            ),
            "UserDryingHour": GeckoByteStructAccessor(
                self.struct, "UserDryingHour", 263, "ALL"
            ),
            "UserDryingMinute": GeckoByteStructAccessor(
                self.struct, "UserDryingMinute", 264, "ALL"
            ),
            "UserDryingSecond": GeckoByteStructAccessor(
                self.struct, "UserDryingSecond", 265, "ALL"
            ),
            "UserDryingDelay": GeckoEnumStructAccessor(
                self.struct,
                "UserDryingDelay",
                266,
                0,
                [
                    "0",
                    "",
                    "",
                    "",
                    "",
                    "5",
                    "",
                    "",
                    "",
                    "",
                    "10",
                    "",
                    "",
                    "",
                    "",
                    "15",
                    "",
                    "",
                    "",
                    "",
                    "20",
                    "",
                    "",
                    "",
                    "",
                    "25",
                    "",
                    "",
                    "",
                    "",
                    "30",
                    "",
                    "",
                    "",
                    "",
                    "35",
                    "",
                    "",
                    "",
                    "",
                    "40",
                    "",
                    "",
                    "",
                    "",
                    "45",
                    "",
                    "",
                    "",
                    "",
                    "50",
                    "",
                    "",
                    "",
                    "",
                    "55",
                    "",
                    "",
                    "",
                    "",
                    "60",
                ],
                None,
                63,
                "ALL",
            ),
            "KeypadID": GeckoWordStructAccessor(self.struct, "KeypadID", 267, None),
            "KeypadRev": GeckoWordStructAccessor(self.struct, "KeypadRev", 269, None),
            "BlowerState": GeckoEnumStructAccessor(
                self.struct,
                "BlowerState",
                271,
                None,
                ["PAUSE", "RUN", "STATE10"],
                None,
                None,
                None,
            ),
            "PurgeStart": GeckoEnumStructAccessor(
                self.struct,
                "PurgeStart",
                262,
                1,
                ["DEACTIVATE", "ACTIVATE"],
                None,
                None,
                None,
            ),
            "PurgeDelayTimer": GeckoEnumStructAccessor(
                self.struct,
                "PurgeDelayTimer",
                262,
                2,
                [
                    "0",
                    "1",
                    "2",
                    "3",
                    "4",
                    "5",
                    "6",
                    "7",
                    "8",
                    "9",
                    "10",
                    "11",
                    "12",
                    "13",
                    "14",
                    "15",
                    "16",
                    "17",
                    "18",
                    "19",
                    "20",
                    "21",
                    "22",
                    "23",
                    "24",
                    "25",
                    "26",
                    "27",
                    "28",
                    "29",
                    "30",
                    "31",
                    "32",
                    "33",
                    "34",
                    "35",
                    "36",
                    "37",
                    "38",
                    "39",
                    "40",
                    "41",
                    "42",
                    "43",
                    "44",
                    "45",
                    "46",
                    "47",
                    "48",
                    "49",
                    "50",
                    "51",
                    "52",
                    "53",
                    "54",
                    "55",
                    "56",
                    "57",
                    "58",
                    "59",
                    "60",
                ],
                None,
                None,
                None,
            ),
            "PurgeStandby": GeckoEnumStructAccessor(
                self.struct, "PurgeStandby", 266, 6, ["STOP", "START"], None, None, None
            ),
            "PowerState": GeckoEnumStructAccessor(
                self.struct, "PowerState", 283, None, ["OFF", "ON"], None, None, None
            ),
            "KeypadLock": GeckoEnumStructAccessor(
                self.struct, "KeypadLock", 284, None, ["OFF", "ON"], None, None, None
            ),
            "BootID": GeckoWordStructAccessor(self.struct, "BootID", 272, None),
            "BootRev": GeckoWordStructAccessor(self.struct, "BootRev", 274, None),
            "PackCoreID": GeckoWordStructAccessor(self.struct, "PackCoreID", 276, None),
            "PackCoreRev": GeckoWordStructAccessor(
                self.struct, "PackCoreRev", 278, None
            ),
            "PackCoreRel": GeckoByteStructAccessor(
                self.struct, "PackCoreRel", 280, None
            ),
            "PackType": GeckoEnumStructAccessor(
                self.struct,
                "PackType",
                281,
                None,
                [
                    "Unknown",
                    "inXE",
                    "MasIBC",
                    "MIA",
                    "DJS4",
                    "inClear",
                    "inXM",
                    "K600",
                    "inTerface",
                    "inTouch",
                    "inYT",
                ],
                None,
                None,
                None,
            ),
            "PackMemRange": GeckoEnumStructAccessor(
                self.struct,
                "PackMemRange",
                282,
                0,
                ["16K", "32K", "48K", "64K"],
                None,
                4,
                None,
            ),
        }
